#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <qapplication.h>
#include <qclipboard.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qlistbox.h>
#include <qlistview.h>
#include <qlayout.h>
#include <qarray.h>
#include <qstrlist.h>
#include <qtabwidget.h>
#include <qfont.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qpushbutton.h>
#include <qstring.h>
#include <qtextcodec.h>
#include <qdir.h>
#include <qimage.h>
#include <qcopenvelope_qws.h>
#include <libkakasi.h>
#include <eb.h>
#include <sldlgwait.h>

#include "qbookentry.h"
#include "qbookbrowser.h"
#include "qhistorylineedit.h"

#include <slmisc.h>

#include "zpixmap.h"

#include "zten.h"
#include "ztenext.h"
#include "ztensetdict.h"
#include "ztenbook.h"
#include "ztencandidate.h"
#include "ztenbookmark.h"

#include "link1.xpm"
#include "link2.xpm"
#include "link3.xpm"
#include "link4.xpm"
#include "link5.xpm"
#include "link6.xpm"
#include "link7.xpm"
#include "link8.xpm"
#include "link9.xpm"
#include "link0.xpm"

static const char* const image0_data[] = { 
"16 16 6 1",
". c None",
"# c #000000",
"b c #0000ff",
"d c #c5c2c5",
"c c #c5c2ff",
"a c #ffffff",
".###########....",
"#aaaaaaaaaaa#...",
"#aaaaaaaabca#...",
"#aaaaaaaaaba#...",
"#aaaad###daa#...",
"#aaad#aaa#da#...",
"#aaa#acaca#a#...",
"#aaa#abbaa#a#...",
"#aaa#aabca#a#...",
"#aaad#aaa##a#...",
"#bcaad####a##...",
"#abaaaaaaa#a#...",
"#aaaaaaaaaa#a#..",
"#aabcaaaaaaa#a#.",
".###########.#a#",
"..............#."};

static const char* const image1_data[] = { 
"22 22 3 1",
". c None",
"# c #000000",
"a c #ffffff",
"......................",
"......................",
"......................",
"......................",
".##############.......",
".###############......",
".################.....",
".###aa#####aa#####....",
".###aaa###aaa######...",
".####aaa#aaa########..",
".#####aaaaa##########.",
".######aaa############",
".#####aaaaa##########.",
".####aaa#aaa########..",
".###aaa###aaa######...",
".###aa#####aa#####....",
".################.....",
".###############......",
".##############.......",
"......................",
"......................",
"......................"};


int scales [] = {50, 75, 100, 150, 200};

#define STR_SEARCH_HEAD  tr("Head  ")
#define STR_SEARCH_TAIL  tr("Tail  ")
#define STR_SEARCH_EXACT tr("Exact ")
#define STR_SEARCH_COND  tr("Cond  ")
#define STR_SEARCH_MULTI tr("Multi ")
#define STR_SEARCH_HEAD_P  tr("Head* ")
#define STR_SEARCH_TAIL_P  tr("Tail* ")
#define STR_SEARCH_COND_P  tr("Cond* ")

#define MARKLIST_MAX_NUM 200
#define BOOKMARK_DISP_MAX_LEN 12
#define BACK_ACTION_MAX_COUNT 20
#define SEARCH_WORD_MAX_LENGTH 30

QBookHitItem::QBookHitItem (const QString &label, QBookEntry *be,
			    int hit_pos)
  : SlListBoxRichText (label, label)
{
  this->be = be;
  this->hit_pos = hit_pos;
}

QBookHitItem::~QBookHitItem ()
{
}

DictInfo::DictInfo()
{
	reset();
}

void DictInfo::reset()
{
	path = "";
	subpos = 0;
	selected = FALSE;
	title = "";
	name = "";
	orgname = "";
	extension = "";
	appendix = "";
	memory = "";
}

BookMark::BookMark()
{
	reset();
}

void BookMark::reset()
{
	entry = "";
	book_title = "";
	pos.page = 0;
	pos.offset = 0;
}

Zten::Zten(QWidget* parent, const char* name, WFlags fl)
    : QWidget( parent, name, fl )
{

  QMimeSourceFactory::defaultFactory ()->setImage ("link1_xpm",
						    QImage (link1_xpm));
  QMimeSourceFactory::defaultFactory ()->setImage ("link2_xpm",
						    QImage (link2_xpm));
  QMimeSourceFactory::defaultFactory ()->setImage ("link3_xpm",
						    QImage (link3_xpm));
  QMimeSourceFactory::defaultFactory ()->setImage ("link4_xpm",
						    QImage (link4_xpm));
  QMimeSourceFactory::defaultFactory ()->setImage ("link5_xpm",
						    QImage (link5_xpm));
  QMimeSourceFactory::defaultFactory ()->setImage ("link6_xpm",
						    QImage (link6_xpm));
  QMimeSourceFactory::defaultFactory ()->setImage ("link7_xpm",
						    QImage (link7_xpm));
  QMimeSourceFactory::defaultFactory ()->setImage ("link8_xpm",
						    QImage (link8_xpm));
  QMimeSourceFactory::defaultFactory ()->setImage ("link9_xpm",
						    QImage (link9_xpm));
  QMimeSourceFactory::defaultFactory ()->setImage ("link0_xpm",
						    QImage (link0_xpm));
  eb_initialize_library();

  config = new Config ("zdict");

  displayMode = -1;
  displayMode2 = -1;

  init();

  books.setAutoDelete (TRUE);
  ms_list.setAutoDelete (TRUE);

  current_book = NULL;
  history.resize (0);
  history_current = -1;

  combiSearchType = 0;
  isDictChanged = FALSE;
  isBookMarkShown = FALSE;
  hasFocusLineEdit = FALSE;
  isNaviEnable = FALSE;

  cur_font_id = 0;
  bookOpened = FALSE;  
  current_entrylist = EntryList;
  backCount = 1;
  isSearching = false;
  isSetCandidate = false;
  isGaijiDisp = FALSE;

  current_bk.reset();
  markList.clear();

  // last selected;
  multi_last_book_ident = QString ();
  multi_last_id = -1;
  menu_last_book_ident = QString ();
 
  connect ((QObject *)LookupButton, SIGNAL (clicked()),
	   this, SLOT (activateAndSearch()));
  connect(SearchMethodComboBox, SIGNAL(activated(int)),
		  this, SLOT(searchMethodChanged(int)));
  connect ((QObject *)CondButton, SIGNAL (clicked()),
	   this, SLOT (backToMultiCond()));

  connect(SearchTypePopup, SIGNAL(activated(int)), this,
		  SLOT(searchTypeSelected(int)) );

  connect ((QObject *)DelButton, SIGNAL (clicked()),
	   this, SLOT (clearLine()));
  connect (LineEdit, SIGNAL (returnPressed ()), this, SLOT (activateAndSearch()));  
  connect (LineEdit, SIGNAL (textChanged (const QString &)),
	   this, SLOT (searchIncremental (const QString &)));
  connect (LineEdit, SIGNAL (focusChange(bool)),
		   this, SLOT (focusInLineEdit(bool)));  

  connect (EntryList, SIGNAL (currentChanged (SlListBoxItem *)), 
	   this, SLOT (itemSelected (SlListBoxItem *)));
  connect (EntryList2, SIGNAL (currentChanged (SlListBoxItem *)), 
	   this, SLOT (itemSelected2 (SlListBoxItem *)));
  
  connect (MultiSearchCombo, SIGNAL (activated (int)),
	   this, SLOT (multiSelected (int)));
  connect (MenuSearch, SIGNAL (activated (int)),
	   this, SLOT (menuSelected (int)));  

  connect (LineEdit1, SIGNAL (returnPressed ()), this, SLOT (multiSearch()));  
  connect (LineEdit2, SIGNAL (returnPressed ()), this, SLOT (multiSearch()));  
  connect (LineEdit3, SIGNAL (returnPressed ()), this, SLOT (multiSearch()));  
  connect (LineEdit4, SIGNAL (returnPressed ()), this, SLOT (multiSearch()));  
  connect (LineEdit5, SIGNAL (returnPressed ()), this, SLOT (multiSearch()));  
			      
  connect (ContentBrowser, SIGNAL (goTo (EB_Position)),
	   this, SLOT (goTo (EB_Position)));
  connect (ContentBrowser, SIGNAL (playSound (int)),
	   this, SLOT (playSound (int)));
  connect (ContentBrowser, SIGNAL (playMpeg (int)),
	   this, SLOT (playMpeg (int)));

  connect (ContentBrowser2, SIGNAL (goTo (EB_Position)),
	   this, SLOT (goTo (EB_Position)));
  connect (ContentBrowser2, SIGNAL (playSound (int)),
	   this, SLOT (playSound (int)));
  connect (ContentBrowser2, SIGNAL (playMpeg (int)),
	   this, SLOT (playMpeg (int)));

  connect (ContentBrowser3, SIGNAL (goTo (EB_Position)),
	   this, SLOT (linkJump3 (EB_Position)));
  connect (ContentBrowser3, SIGNAL (playSound (int)),
	   this, SLOT (playSound (int)));
  connect (ContentBrowser3, SIGNAL (playMpeg (int)),
	   this, SLOT (playMpeg (int)));

  connect (MultiCandidate1, SIGNAL (clicked ()),
	   this, SLOT (multiCandidate1 ()));
  connect (MultiCandidate2, SIGNAL (clicked ()),
	   this, SLOT (multiCandidate2 ()));
  connect (MultiCandidate3, SIGNAL (clicked ()),
	   this, SLOT (multiCandidate3 ()));
  connect (MultiCandidate4, SIGNAL (clicked ()),
	   this, SLOT (multiCandidate4 ()));
  connect (MultiCandidate5, SIGNAL (clicked ()),
	   this, SLOT (multiCandidate5 ()));

  connect(LineEdit1, SIGNAL( textChanged( const QString& ) ),
		  this, SLOT( slotTextChanged1( const QString&) ) );
  connect(LineEdit2, SIGNAL( textChanged( const QString& ) ),
		  this, SLOT( slotTextChanged2( const QString&) ) );
  connect(LineEdit3, SIGNAL( textChanged( const QString& ) ),
		  this, SLOT( slotTextChanged3( const QString&) ) );
  connect(LineEdit4, SIGNAL( textChanged( const QString& ) ),
		  this, SLOT( slotTextChanged4( const QString&) ) );
  connect(LineEdit5, SIGNAL( textChanged( const QString& ) ),
		  this, SLOT( slotTextChanged5( const QString&) ) );

  candidate1 = QString ();
  candidate2 = QString ();
  candidate3 = QString ();
  candidate4 = QString ();
  candidate5 = QString ();

  EntryList->installEventFilter (this);
  ContentBrowser->installEventFilter (this);
  EntryList2->installEventFilter (this);
  ContentBrowser2->installEventFilter (this);
  ContentBrowser3->installEventFilter (this);
  
  toggleContentForce (DISP_LIST);

}

Zten::~Zten()
{
  writeConfig ();
  delete config;
//  eb_initialize_library();
  eb_finalize_library();
}

void Zten::init()
{

	initMainSearch();
	initMultiSearch();
	initMenuSearch();

	EntryList->setHScrollBarMode( QScrollView::AlwaysOff );
	ContentBrowser->setHScrollBarMode( SLTextBrowser::Auto );

	EntryList2->setHScrollBarMode( QScrollView::AlwaysOff );
	ContentBrowser2->setHScrollBarMode( SLTextBrowser::Auto );

	BookContentLayout->setStretchFactor(EntryList, 1);
	BookContentLayout->setStretchFactor(ContentBrowser, 2);

	BookContentLayout2->setStretchFactor(EntryList2, 1);
	BookContentLayout2->setStretchFactor(ContentBrowser2, 2);

	CondButton->hide();
	updateMultiSearchTools(FALSE, FALSE);

	EntryList2->hide();
	ContentBrowser2->hide();

	MenuSearch->hide();
	ContentBrowser3->hide();

}

void Zten::initMainSearch()
{

    QPixmap image0( ( const char** ) image0_data );
    QPixmap image1( ( const char** ) image1_data );

    setIcon( image0 );
    ZtenBaseLayout = new QGridLayout( this ); 
    ZtenBaseLayout->setSpacing( 2 );
    ZtenBaseLayout->setMargin( 0 );

    MainLayout = new QHBoxLayout; 
    MainLayout->setSpacing( 2 );
    MainLayout->setMargin( 0 );

    DelButton = new QPushButton( this, "DelButton" );
    DelButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1,
		(QSizePolicy::SizeType)0, DelButton->sizePolicy().hasHeightForWidth() ) );
    DelButton->setText( tr( "" ) );
    DelButton->setPixmap( image1 );
    MainLayout->addWidget( DelButton );

    CondButton = new QPushButton( this, "CondButton" );
    CondButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1,
		(QSizePolicy::SizeType)0, DelButton->sizePolicy().hasHeightForWidth() ) );
    CondButton->setText( tr( "goto Input Form" ) );
    MainLayout->addWidget( CondButton );

    LineEdit = new QHistoryLineEdit( this, "LineEdit" );
    LineEdit->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7,
		(QSizePolicy::SizeType)0, LineEdit->sizePolicy().hasHeightForWidth() ) );
    LineEdit->setMinimumSize( QSize( 0, 18 ) );
    LineEdit->setFocusPolicy( QHistoryLineEdit::StrongFocus );
	LineEdit->lineEdit()->setMaxLength(SEARCH_WORD_MAX_LENGTH);
    MainLayout->addWidget( LineEdit );

	SearchMethodComboBox = new QComboBox( FALSE, this, "SearchMethodComboBox" );
	SearchMethodComboBox->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1,
		(QSizePolicy::SizeType)0, SearchMethodComboBox->sizePolicy().hasHeightForWidth() ) );
	MainLayout->addWidget( SearchMethodComboBox );

	SearchTypeButton = new QPushButton(this);
	SearchTypeButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1,
		(QSizePolicy::SizeType)0, SearchTypeButton->sizePolicy().hasHeightForWidth() ) );

	SearchTypePopup = new QPopupMenu(SearchTypeButton);
	SearchTypeButton->setPopup(SearchTypePopup);
	SearchTypePopup->insertItem(STR_SEARCH_HEAD, SEARCH_HEAD);
	SearchTypePopup->insertItem(STR_SEARCH_TAIL, SEARCH_TAIL);
	SearchTypePopup->insertItem(STR_SEARCH_EXACT, SEARCH_EXACT);

	SearchTypeButton->setText(STR_SEARCH_HEAD);
	MainLayout->addWidget( SearchTypeButton );

	LookupButton = new QPushButton( this, "LookupButton" );
	LookupButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1,
		(QSizePolicy::SizeType)0, LookupButton->sizePolicy().hasHeightForWidth() ) );
	LookupButton->setText( tr( "Search" ) );
	MainLayout->addWidget( LookupButton );

	ZtenBaseLayout->addLayout( MainLayout, 0, 0 );

	BookContentLayout = new QHBoxLayout; 
	BookContentLayout->setSpacing( 2 );
	BookContentLayout->setMargin( 0 );

	EntryList = new SlListBox( this, "EntryList" );
	EntryList->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3,
		(QSizePolicy::SizeType)3, EntryList->sizePolicy().hasHeightForWidth() ) );
	BookContentLayout->addWidget( EntryList );

	ContentBrowser = new QBookBrowser( this, "ContentBrowser" );
	ContentBrowser->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7,
		(QSizePolicy::SizeType)7, ContentBrowser->sizePolicy().hasHeightForWidth() ) );
	BookContentLayout->addWidget( ContentBrowser );

	ZtenBaseLayout->addLayout( BookContentLayout, 1, 0 );

	SearchMethodComboBox->insertItem (tr ("Word"));
	SearchMethodComboBox->insertItem (tr ("Multi"));
	SearchMethodComboBox->insertItem (tr ("Cond"));
	SearchMethodComboBox->insertItem (tr ("Combine"));
	SearchMethodComboBox->insertItem (tr ("Menu"));

}

void Zten::initMultiSearch()
{

	MultiSearchTabLayout = new QGridLayout; 

	MultiSearchTabLayout->setSpacing( 2 );
	MultiSearchTabLayout->setMargin( 0 );

	MultiSearchCombo  = new QComboBox( FALSE, this, "MultiSearch" );
	MultiSearchTabLayout->addWidget( MultiSearchCombo,0,0 );

	Layout_m1 = new QHBoxLayout; 
	Layout_m1->setSpacing( 6 );
	Layout_m1->setMargin( 0 );

	MultiLabel1 = new QLabel( this, "MultiLabel1" );
	MultiLabel1->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7,
		(QSizePolicy::SizeType)1, MultiLabel1->sizePolicy().hasHeightForWidth() ) );
	MultiLabel1->setMinimumSize( QSize( 100, 0 ) );
	MultiLabel1->setText( tr( "" ) );
	MultiLabel1->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );
	Layout_m1->addWidget( MultiLabel1 );

	LineEdit1 = new QLineEdit( this, "LineEdit1" );
	Layout_m1->addWidget( LineEdit1 );

	MultiCandidate1 = new QPushButton( this, "MultiCandidate1" );
	MultiCandidate1->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)4,
		(QSizePolicy::SizeType)4, MultiCandidate1->sizePolicy().hasHeightForWidth() ) );
	MultiCandidate1->setMinimumSize( QSize( 0, 0 ) );
	MultiCandidate1->setMaximumSize( QSize( 45, 32767 ) );
	MultiCandidate1->setText( tr( "..." ) );
	Layout_m1->addWidget( MultiCandidate1 );
	MultiSearchTabLayout->addLayout( Layout_m1,1,0 );

	Layout_m2 = new QHBoxLayout; 
	Layout_m2->setSpacing( 6 );
	Layout_m2->setMargin( 0 );

	MultiLabel2 = new QLabel( this, "MultiLabel2" );
	MultiLabel2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7,
		(QSizePolicy::SizeType)1, MultiLabel2->sizePolicy().hasHeightForWidth() ) );
	MultiLabel2->setMinimumSize( QSize( 100, 0 ) );
	MultiLabel2->setText( tr( "" ) );
	MultiLabel2->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );
	Layout_m2->addWidget( MultiLabel2 );

	LineEdit2 = new QLineEdit( this, "LineEdit2" );
	LineEdit2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7,
		(QSizePolicy::SizeType)0, LineEdit2->sizePolicy().hasHeightForWidth() ) );
	Layout_m2->addWidget( LineEdit2 );

	MultiCandidate2 = new QPushButton( this, "MultiCandidate2" );
	MultiCandidate2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)4,
		(QSizePolicy::SizeType)4, MultiCandidate2->sizePolicy().hasHeightForWidth() ) );
	MultiCandidate2->setMaximumSize( QSize( 45, 32767 ) );
	MultiCandidate2->setText( tr( "..." ) );
	Layout_m2->addWidget( MultiCandidate2 );
	MultiSearchTabLayout->addLayout( Layout_m2,2,0 );

	Layout_m3 = new QHBoxLayout; 
	Layout_m3->setSpacing( 6 );
	Layout_m3->setMargin( 0 );

	MultiLabel3 = new QLabel( this, "MultiLabel3" );
	MultiLabel3->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7,
		(QSizePolicy::SizeType)1, MultiLabel3->sizePolicy().hasHeightForWidth() ) );
	MultiLabel3->setMinimumSize( QSize( 100, 0 ) );
	MultiLabel3->setText( tr( "" ) );
	MultiLabel3->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );
	Layout_m3->addWidget( MultiLabel3 );

	LineEdit3 = new QLineEdit( this, "LineEdit3" );
	Layout_m3->addWidget( LineEdit3 );

	MultiCandidate3 = new QPushButton( this, "MultiCandidate3" );
	MultiCandidate3->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)4,
		(QSizePolicy::SizeType)4, MultiCandidate3->sizePolicy().hasHeightForWidth() ) );
	MultiCandidate3->setMaximumSize( QSize( 45, 32767 ) );
	MultiCandidate3->setText( tr( "..." ) );
	Layout_m3->addWidget( MultiCandidate3 );
	MultiSearchTabLayout->addLayout( Layout_m3,3,0 );

	Layout_m4 = new QHBoxLayout; 
	Layout_m4->setSpacing( 6 );
	Layout_m4->setMargin( 0 );

	MultiLabel4 = new QLabel( this, "MultiLabel4" );
	MultiLabel4->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7,
		(QSizePolicy::SizeType)1, MultiLabel4->sizePolicy().hasHeightForWidth() ) );
	MultiLabel4->setMinimumSize( QSize( 100,0 ) );
	MultiLabel4->setText( tr( "" ) );
	MultiLabel4->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );
	Layout_m4->addWidget( MultiLabel4 );

	LineEdit4 = new QLineEdit( this, "LineEdit4" );
	Layout_m4->addWidget( LineEdit4 );

	MultiCandidate4 = new QPushButton( this, "MultiCandidate4" );
	MultiCandidate4->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)4,
		(QSizePolicy::SizeType)4, MultiCandidate4->sizePolicy().hasHeightForWidth() ) );
	MultiCandidate4->setMaximumSize( QSize( 45, 32767 ) );
	MultiCandidate4->setText( tr( "..." ) );
	Layout_m4->addWidget( MultiCandidate4 );
	MultiSearchTabLayout->addLayout( Layout_m4,4,0 );

	Layout_m5 = new QHBoxLayout; 
	Layout_m5->setSpacing( 6 );
	Layout_m5->setMargin( 0 );

	MultiLabel5 = new QLabel( this, "MultiLabel5" );
	MultiLabel5->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7,
		(QSizePolicy::SizeType)1, MultiLabel5->sizePolicy().hasHeightForWidth() ) );
	MultiLabel5->setMinimumSize( QSize( 100, 0 ) );
	MultiLabel5->setText( tr( "" ) );
	MultiLabel5->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );
	Layout_m5->addWidget( MultiLabel5 );

	LineEdit5 = new QLineEdit( this, "LineEdit5" );
	Layout_m5->addWidget( LineEdit5 );

	MultiCandidate5 = new QPushButton( this, "MultiCandidate5" );
	MultiCandidate5->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)4,
		(QSizePolicy::SizeType)4, MultiCandidate5->sizePolicy().hasHeightForWidth() ) );
	MultiCandidate5->setMinimumSize( QSize( 0, 0 ) );
	MultiCandidate5->setMaximumSize( QSize( 45, 32767 ) );
	MultiCandidate5->setText( tr( "..." ) );
	Layout_m5->addWidget( MultiCandidate5 );
	MultiSearchTabLayout->addLayout( Layout_m5,5,0 );

	Layout_m6 = new QHBoxLayout; 
	Layout_m6->setSpacing( 6 );
	Layout_m6->setMargin( 45 );

	MultiLabel6 = new QLabel( this, "MultiLabel6" );
	MultiLabel6->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7,
		(QSizePolicy::SizeType)1, MultiLabel6->sizePolicy().hasHeightForWidth() ) );
	MultiLabel6->setMinimumSize( QSize( 100, 0 ) );
	MultiLabel6->setText( tr( "" ) );
	MultiLabel6->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );
	Layout_m6->addWidget( MultiLabel6 );

	gaijiBrowser = new QBookBrowser( this, "gaijiBrowser" );
	gaijiBrowser->setMinimumSize( QSize( 150, 0 ) );
	gaijiBrowser->setMaximumSize( QSize( 150, 40 ) );

	gaijiBrowser->setVScrollBarMode( SLTextBrowser::AlwaysOff );
	Layout_m6->addWidget( gaijiBrowser );
	MultiSearchTabLayout->addLayout( Layout_m6,6,0 );
	gaijiBrowser->hide();

	QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
	MultiSearchTabLayout->addItem( spacer,7,0 );

	ZtenBaseLayout->addLayout( MultiSearchTabLayout, 1, 0 );

	BookContentLayout2 = new QHBoxLayout; 
	BookContentLayout2->setSpacing( 2 );
	BookContentLayout2->setMargin( 0 );

	EntryList2 = new SlListBox( this, "EntryList2" );
	EntryList->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3,
		(QSizePolicy::SizeType)3, EntryList->sizePolicy().hasHeightForWidth() ) );

	BookContentLayout2->addWidget( EntryList2 );

	ContentBrowser2 = new QBookBrowser( this, "ContentBrowser2" );
	ContentBrowser2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7,
		(QSizePolicy::SizeType)7, ContentBrowser2->sizePolicy().hasHeightForWidth() ) );
	BookContentLayout2->addWidget( ContentBrowser2 );

	ZtenBaseLayout->addLayout( BookContentLayout2, 1, 0 );

	updateMultiSearchTools(false, false);
}

void Zten::initMenuSearch()
{
	 MenuTabLayout = new QVBoxLayout; 
	 MenuTabLayout->setSpacing( 2 );
	 MenuTabLayout->setMargin( 0 );
	 MenuSearch = new QComboBox( FALSE, this, "MenuSearch" );
	 MenuTabLayout->addWidget( MenuSearch );

	 ContentBrowser3 = new QBookBrowser( this, "ContentBrowser3" );
	 ContentBrowser3->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7,
		(QSizePolicy::SizeType)7, ContentBrowser3->sizePolicy().hasHeightForWidth() ) );
	 MenuTabLayout->addWidget( ContentBrowser3 );

	 ZtenBaseLayout->addLayout( MenuTabLayout, 1, 0 );

}

void
Zten::readConfig ()
{

	config->setGroup ("Main");
	cur_font_id = config->readNumEntry ("FontSizeId");
	cur_font_id -= 2;
	if( (cur_font_id < -2) || (cur_font_id > 2) )
		cur_font_id = 0;
	cur_scale = config->readNumEntry ("ImageScale");
	cur_scale -= 2;
	if( (cur_scale < -2) || (cur_scale > 2) )
		cur_scale = 0;

	if((currentMethod = config->readNumEntry ("CurrentMethod")) == -1)
		currentMethod = 0;

   int type = config->readNumEntry ("SearchType");
   if(type <= 0)
	   type = SEARCH_HEAD;
   else
	   type += 1;
   searchTypeSelected(type);

   multi_last_book_ident = config->readEntry ("LastMultiBook");
   multi_last_id = config->readNumEntry ("LastMultiId");
   menu_last_book_ident = config->readEntry ("LastMenuBook");

   if (!config->hasKey ("IncrementalSearch"))
	 {
	   // Default is false;
	   incremental_mode = false;
	 }
   else
	 {
	   incremental_mode = config->readBoolEntry ("IncrementalSearch");
	 }
   emit incrementalIsSet (incremental_mode);

   if (!config->hasKey ("Continuous")) {
	   // Default is on;
	   continuous_mode = false;
   } else {
	   continuous_mode = config->readBoolEntry ("Continuous");
   }

   emit continuousIsSet (continuous_mode);

   config->setGroup ("Books");
   EB_Book book;
   eb_initialize_book (&book);

   dictList.clear();

   for (int i = 0; i < config->readNumEntry ("NumberOfBooks"); i++)
	 {
		 DictInfo *dictInfo = new DictInfo();

		 QString entryName;

	   entryName.sprintf ("BookPath%i", i + 1);
	   dictInfo->setPath(config->readEntry (entryName));
	   entryName.sprintf ("BookSubPos%i", i + 1);
	   dictInfo->setSubpos(config->readNumEntry (entryName));
	   entryName.sprintf ("BookSelected%i", i + 1);
	   dictInfo->setSelected(config->readBoolEntry (entryName));
	   entryName.sprintf ("BookTitle%i", i + 1);
	   dictInfo->setTitle(config->readEntry (entryName));
	   entryName.sprintf ("BookName%i", i + 1);
	   QString name = config->readEntry (entryName);
	   dictInfo->setOrgName(name);
	   dictInfo->setName(QString::fromUtf8(name));
	   entryName.sprintf ("BookMemory%i", i + 1);
	   dictInfo->setMemory(config->readEntry (entryName));
	   entryName.sprintf ("BookDirectory%i", i + 1);
	   dictInfo->setDirectory(config->readEntry (entryName));
	   dictList.append(dictInfo);

	 }

   config->setGroup ("History");

   int histNum = config->readNumEntry ("NumberOfHistory");
   if(histNum > 0) {
	   for(int i=0; i<histNum; i++) {
		   QString entryName;
		   entryName.sprintf ("Histroy%i",i);
		   QString tmp = config->readEntry (entryName);
		   if(!tmp.isEmpty()) {
			   LineEdit->insertItem(tmp, i);
		   }
   }
	   LineEdit->setFocus ();
	   LineEdit->setEditText ("");
   }

   config->setGroup ("BookMark");

   int markNum = config->readNumEntry ("NumberOfBookMark");
   if(markNum > 0) {
	   for(int i=1; i<markNum+1; i++) {
		   BookMark *bk = new BookMark();
		   EB_Position pos;
		   QString entryName;
		   entryName.sprintf ("BkEentry%i",i);
		   QString tmp = config->readEntry (entryName);
		   bk->setEntry(tmp);
		   entryName.sprintf ("BkBookTitley%i",i);
		   tmp = config->readEntry (entryName);
		   bk->setBookTitle(tmp);
		   entryName.sprintf ("BkPosPage%i",i);
		   pos.page = config->readNumEntry (entryName);
		   entryName.sprintf ("BkPosOffset%i",i);
		   pos.offset = config->readNumEntry (entryName);
		   bk->setPosition(pos);
		   entryName.sprintf ("BkHeadPosPage%i",i);
		   pos.page = config->readNumEntry (entryName);
		   entryName.sprintf ("BkHeadPosOffset%i",i);
		   pos.offset = config->readNumEntry (entryName);
		   bk->setHeadPosition(pos);

		   markList.append(bk);
	   }
   }

   eb_finalize_book (&book);

   setCurrentMethod(currentMethod);
}

void
Zten::writeConfig ()
{
  config->setGroup ("Main");
  config->writeEntry ("FontSizeId", cur_font_id+2);
  config->writeEntry ("CurrentMethod", currentMethod);
  config->writeEntry ("ImageScale", cur_scale+2);
  config->writeEntry ("SearchType", getSearchType());
  config->writeEntry ("LastMultiBook", multi_last_book_ident);
  config->writeEntry ("LastMultiId", multi_last_id);
  config->writeEntry ("LastMenuBook", menu_last_book_ident);
  config->writeEntry ("IncrementalSearch", incremental_mode);
  config->writeEntry ("Continuous", continuous_mode);
  
  config->setGroup ("Books");
  int number = 0;

  DictInfo *dictInfo;

  for (dictInfo = dictList.first(); dictInfo != 0;
	   dictInfo = dictList.next()) {

      QString entryName;
      entryName.sprintf ("BookPath%i", number + 1);
      config->writeEntry (entryName, dictInfo->getPath());
      entryName.sprintf ("BookSubPos%i", number + 1);
      config->writeEntry (entryName, dictInfo->getSubpos());
      entryName.sprintf ("BookSelected%i", number + 1);
      config->writeEntry (entryName, dictInfo->getSelected());
      entryName.sprintf ("BookTitle%i", number + 1);
      config->writeEntry (entryName, dictInfo->getTitle());
      entryName.sprintf ("BookName%i", number + 1);
      config->writeEntry (entryName, dictInfo->getOrgName());
      entryName.sprintf ("BookMemory%i", number + 1);
      config->writeEntry (entryName, dictInfo->getMemory());
      entryName.sprintf ("BookDirectory%i", number + 1);
      config->writeEntry (entryName, dictInfo->getDirectory());

	  number++;
  }

  config->writeEntry ("NumberOfBooks", number);

  config->setGroup ("History");

  int histNum = LineEdit->count();
  for(int i=0; i<histNum; i++) {
      QString entryName;
      entryName.sprintf ("Histroy%i",i);
      config->writeEntry (entryName, LineEdit->text(i));
  }
  config->writeEntry ("NumberOfHistory", histNum);

  config->setGroup ("BookMark");

  number = 0;
  BookMark *bk;
  for(bk = markList.first(); bk != 0; bk = markList.next()){
      QString entryName;
      entryName.sprintf ("BkEentry%i",number + 1);
      config->writeEntry (entryName, bk->getEntry());
      entryName.sprintf ("BkBookTitley%i",number + 1);
      config->writeEntry (entryName, bk->getBookTitle());
      entryName.sprintf ("BkPosPage%i",number + 1);
      config->writeEntry (entryName, bk->getPosition().page);
      entryName.sprintf ("BkPosOffset%i",number + 1);
      config->writeEntry (entryName, bk->getPosition().offset);
      entryName.sprintf ("BkHeadPosPage%i",number + 1);
      config->writeEntry (entryName, bk->getHeadPosition().page);
      entryName.sprintf ("BkHeadPosOffset%i",number + 1);
      config->writeEntry (entryName, bk->getHeadPosition().offset);

	  number++;
  }
  config->writeEntry ("NumberOfBookMark", number);


  config->write();

}


void Zten::delayBuild()
{
	changeLayout(MAIN_SEARCH_METHOD_NO);
	setCurrentFont (cur_font_id);
	setImageScale (cur_scale);
}


void
Zten::entryListEscape ()
{
	LineEdit->setFocus();
}

void
Zten::entryListUp ()
{
	SlListBoxItem *item;
	int i = current_entrylist->currentItem ();
	if (i >= 0) {
		if ((item = current_entrylist->item(i)->prev())) {
			current_entrylist->setCurrentItem (item);
		}
    }
}

void
Zten::entryListDown ()
{

	SlListBoxItem *item;
	int i = current_entrylist->currentItem ();
	if (i >= 0) {
		if ((item = current_entrylist->item(i)->next())) {	
			current_entrylist->setCurrentItem (item);
		}
    }  
}

void
Zten::contentLineDown ()
{
	current_browser->scrollBy (0, cur_font.pixelSize());
}

void
Zten::contentLineUp ()
{

	if( (current_browser->contentsY() == 0) && continuous_mode) {
		backAction();
		return;
	}

	current_browser->scrollBy (0, -cur_font.pixelSize());
}

void
Zten::contentPageUp ()
{
	ContentBrowser->scrollBy (0, ContentBrowser->height () * -8 / 10);
}

void
Zten::contentPageDown ()
{
	ContentBrowser->scrollBy (0, ContentBrowser->height () * 8 / 10);
}

void
Zten::contentPageUp2 ()
{
	ContentBrowser2->scrollBy (0, ContentBrowser2->height () * -8 / 10);
}

void
Zten::contentPageDown2 ()
{
      ContentBrowser2->scrollBy (0, ContentBrowser2->height () * 8 / 10);
}

void
Zten::contentPageUp3 ()
{
  ContentBrowser3->scrollBy (0, ContentBrowser3->height () * -8 / 10);
}

void
Zten::contentPageDown3 ()
{
  ContentBrowser3->scrollBy (0, ContentBrowser3->height () * 8 / 10);
}

void
Zten::toggleDirection ()
{
  if (BookContentLayout->direction () == QBoxLayout::LeftToRight)
    {
      EntryList->setMinimumSize(QSize(ContentBrowser->width (),
				      ContentBrowser->height () / 3));
      BookContentLayout->setDirection (QBoxLayout::TopToBottom);
    }
  else
    {
      EntryList->setMinimumSize(QSize(ContentBrowser->width () / 3,
				      ContentBrowser->height ()));      
      BookContentLayout->setDirection (QBoxLayout::LeftToRight);
    }
}

void
Zten::toggleDirection2 ()
{
  if (BookContentLayout2->direction () == QBoxLayout::LeftToRight)
    {
      EntryList2->setMinimumSize(QSize(ContentBrowser2->width (),
				       ContentBrowser2->height () / 3));
      BookContentLayout2->setDirection (QBoxLayout::TopToBottom);
    }
  else
    {
      EntryList2->setMinimumSize(QSize(ContentBrowser2->width () / 3,
				       ContentBrowser2->height ()));      
      BookContentLayout2->setDirection (QBoxLayout::LeftToRight);
    }
}

void
Zten::resizeEvent (QResizeEvent* e)
{
  QSize s = e->size ();
    if (SlMisc::isLandscape()) {
		BookContentLayout->setDirection (QBoxLayout::LeftToRight);
		BookContentLayout2->setDirection (QBoxLayout::LeftToRight);
    }
	else {
		BookContentLayout->setDirection (QBoxLayout::TopToBottom);      
		BookContentLayout2->setDirection (QBoxLayout::TopToBottom);
    }

	// FIXME: hard coding.
	EntryList->setMinimumSize(QSize(s.width () / 3,
				  s.height () / 4));
	EntryList2->setMinimumSize(QSize(s.width () / 3,
				   s.height () / 4));

#if 1 // adjust view size
	if (current_browser) {
	  current_browser->resizeContents(0,0);
	  current_browser->setText(current_browser->text());
	}
#endif

}

void
Zten::loopContent ()
{

	if(currentMethod == MAIN_SEARCH_METHOD_NO ||
	   currentMethod == CONDITION_SEARCH_METHOD_NO ||
	   currentMethod == COMBINATION_SEARCH_METHOD_NO ) {
		if(displayMode ==DISP_LIST_AND_CONTENT)
			displayMode =DISP_CONTENT;
		else if(displayMode ==DISP_CONTENT)
			displayMode =DISP_LIST;
		else if(displayMode ==DISP_LIST)
			displayMode =DISP_LIST_AND_CONTENT;

		changeDispContent(displayMode);
	}
	else if(currentMethod == MULTI_SEARCH_METHOD_NO) {
		if(displayMode2 ==DISP_LIST_AND_CONTENT)
			displayMode2 =DISP_CONTENT;
		else if(displayMode2 ==DISP_CONTENT)
			displayMode2 =DISP_LIST;
		else if(displayMode2 ==DISP_LIST)
			displayMode2 =DISP_LIST_AND_CONTENT;

		changeDispContent2(displayMode2);
	}
}

void
Zten::changeDispMode (int dispMode)
{

	if(currentMethod == MAIN_SEARCH_METHOD_NO ||
	   currentMethod == CONDITION_SEARCH_METHOD_NO ||
	   currentMethod == COMBINATION_SEARCH_METHOD_NO ) {
		displayMode = dispMode;
		changeDispContent(dispMode);
	}
	else if(currentMethod == MULTI_SEARCH_METHOD_NO) {
		displayMode2 = dispMode;
		changeDispContent2(dispMode);
	}


}

void
Zten::toggleContent ()
{

	if(displayMode ==DISP_LIST_AND_CONTENT)
		displayMode =DISP_LIST;
	else if(displayMode ==DISP_LIST)
		displayMode =DISP_LIST_AND_CONTENT;
	else if(displayMode ==DISP_CONTENT)
		return;

	changeDispContent(displayMode);
}


void
Zten::changeDispContent (int dispMode)
{

	int i;
	switch(dispMode) {
	case DISP_LIST:
		if (ContentBrowser->hasFocus ())
			EntryList->setFocus();
		ContentBrowser->hide ();
		if(EntryList->isHidden())
			EntryList->show();
		break;
	case DISP_CONTENT:
		i = EntryList->currentItem ();
		if (i >= 0 && redisplay (EntryList->item(i))) {
			if(ContentBrowser->isHidden())
				ContentBrowser->show ();
			ContentBrowser->setFocus ();
			EntryList->hide();
		}
		break;
	case DISP_LIST_AND_CONTENT:
	default:
		if(EntryList->isHidden())
			EntryList->show();
		EntryList->setFocus();
		i = EntryList->currentItem ();
		if (i >= 0 && redisplay (EntryList->item(i))) {
			if(ContentBrowser->isHidden())
				ContentBrowser->show ();
		}

		break;
	}

	emit dispContentChanged(dispMode);

}


void
Zten::toggleContentForce (int dispNo)
{
	if ((dispNo != displayMode) || isBookMarkShown) {
		isBookMarkShown = FALSE;
		displayMode = dispNo;
		changeDispContent(displayMode);
	}
}

void
Zten::toggleContent2 ()
{
	if(displayMode2 ==DISP_LIST_AND_CONTENT)
		displayMode2 =DISP_LIST;
	else if(displayMode2 ==DISP_LIST)
		displayMode2 =DISP_LIST_AND_CONTENT;
	else if(displayMode2 ==DISP_CONTENT)
		return;

	changeDispContent2(displayMode2);
}


void
Zten::changeDispContent2 (int dispMode2)
{
	int i;
	switch(dispMode2) {
	case DISP_LIST:
		if (ContentBrowser2->hasFocus ())
			EntryList2->setFocus();
		ContentBrowser2->hide ();
		if(EntryList2->isHidden())
			EntryList2->show();
		break;
	case DISP_CONTENT:
		i = EntryList2->currentItem ();
		if (i >= 0 && redisplay (EntryList2->item(i))) {
			if(ContentBrowser2->isHidden())
				ContentBrowser2->show ();
			ContentBrowser2->setFocus ();
			EntryList2->hide();
		}
		break;
	case DISP_LIST_AND_CONTENT:
	default:
		if(EntryList2->isHidden())
			EntryList2->show();
		EntryList2->setFocus();
		i = EntryList2->currentItem ();
		if (i >= 0 && redisplay (EntryList2->item(i))) {
			if(ContentBrowser2->isHidden())
				ContentBrowser2->show ();
		}
		break;

	}

	emit dispContentChanged(dispMode2);

}


void
Zten::toggleContentForce2 (int dispNo)
{
	if ((dispNo != displayMode2) || isBookMarkShown) {
		isBookMarkShown = FALSE;
		displayMode2 = dispNo;
		changeDispContent2(displayMode2);
	}

}

void Zten::updateEntryList1(int matchNum)
{
	if(matchNum == 0 ) {
		EntryList->insertItem (new QBookHitItem (tr("No matches"), NULL, -1));
		EntryList->setSelectionMode (SlListBox::NoSelection);

		current_browser->setText("");
		emit updateDownAction(FALSE);
		emit updateBookMarkAction(FALSE);
	}
	else {
		EntryList->setSelectionMode(SlListBox::Single);
		EntryList->setContentsPos(0, 0);

		emit updateDownAction(TRUE);
		emit updateBookMarkAction(TRUE);
	}
}

void Zten::updateEntryList2(int matchNum)
{

	if(EntryList2->isHidden())
		EntryList2->show();
	
	if(matchNum == 0 ) {

		EntryList2->insertItem (new QBookHitItem (tr("No matches"), NULL, -1));
		EntryList2->setSelectionMode (SlListBox::NoSelection);

		current_browser->setText("");

		emit updateDownAction(FALSE);
		emit updateBookMarkAction(FALSE);
	}
	else {
		EntryList2->setSelectionMode(SlListBox::Single);
		EntryList2->setSelected(0, true);

		emit updateDownAction(TRUE);
		emit updateBookMarkAction(TRUE);
	}

}

void
Zten::linkJump (int pos)
{
  EB_Position p = current_book->getLinkPosition (pos);
  if (p.page >= 0) goTo (p);
}

void
Zten::keyPressEvent( QKeyEvent *e )
{

	switch(e->key()){
	case Qt::Key_ZoomIn:
	case Qt::Key_ZoomOut:
		emit zoomSelected(e->key() == Key_ZoomIn);
		e->ignore();
		break;
	default:
	      return;
	}

}

void Zten::setNaviFlag(bool b)
{
	isNaviEnable = b;
}


bool
Zten::eventFilter (QObject *o, QEvent *e)
{

	if (e->type () == QEvent::KeyPress) {

		QKeyEvent *k = (QKeyEvent *)e;

		switch (k->key ()) {
	    case Qt::Key_1:
	    case Qt::Key_2:
	    case Qt::Key_3:
	    case Qt::Key_4:
	    case Qt::Key_5:
	    case Qt::Key_6:
	    case Qt::Key_7:
	    case Qt::Key_8:
	    case Qt::Key_9:
			if(isBookMarkShown) {
				showDlgWait();
				break;
			}
			if(currentMethod == MENU_SEARCH_METHOD_NO)
				linkJump_menu (k->key () - Qt::Key_1);
			else
				linkJump (k->key () - Qt::Key_1);
			break;
	    case Qt::Key_0:
			if(isBookMarkShown) {
				showDlgWait();
				break;
			}
			if(currentMethod == MENU_SEARCH_METHOD_NO)
				linkJump_menu (9);
			else
				linkJump (9);
			break;
	    case Qt::Key_B: // 'B'
			if(isBookMarkShown) {
				showDlgWait();
				break;
			}
			if (k->state () & ShiftButton)
				back ();
			break;
	    case Qt::Key_F:	// 'F'
			if(isBookMarkShown) {
				showDlgWait();
				break;
			}
			if (k->state () & ShiftButton)
				forward ();
			break;

	    case Qt::Key_J:
	    case Qt::Key_N:
	    case Qt::Key_Down:
			if(currentMethod == MENU_SEARCH_METHOD_NO) {
				contentLineDown ();
			} else {
				if(isBookMarkShown) {
					if(current_browser->hasFocus())
						contentLineDown ();
					else
						showDlgWait();
					break;
				}

				if(current_browser->hasFocus())
					contentLineDown ();
				else
					entryListDown ();
			}
			break;

	    case Qt::Key_K:
	    case Qt::Key_P:
	    case Qt::Key_Up:
			if(currentMethod == MENU_SEARCH_METHOD_NO) {
				contentLineUp ();
			} else {
				if(isBookMarkShown) {
					if(current_browser->hasFocus())
						contentLineUp ();
					else
						showDlgWait();
					break;
				}

				if(current_browser->hasFocus())
					contentLineUp ();
				else
					entryListUp ();
			}
			break;

	    case Qt::Key_Q:
			if(currentMethod == MAIN_SEARCH_METHOD_NO ||
			   currentMethod == CONDITION_SEARCH_METHOD_NO ||
			   currentMethod == COMBINATION_SEARCH_METHOD_NO )
				entryListEscape ();
			break;

	    case Qt::Key_V:
			if(currentMethod == MAIN_SEARCH_METHOD_NO ||
			   currentMethod == CONDITION_SEARCH_METHOD_NO ||
			   currentMethod == COMBINATION_SEARCH_METHOD_NO )
				toggleContent ();
			else if(currentMethod == MULTI_SEARCH_METHOD_NO)
				toggleContent2 ();
			break;

	    case Qt::Key_D:
			if (k->state () & ShiftButton) {
				if(currentMethod == MAIN_SEARCH_METHOD_NO ||
				   currentMethod == CONDITION_SEARCH_METHOD_NO ||
				   currentMethod == COMBINATION_SEARCH_METHOD_NO ) {
					toggleDirection ();
				}
				else if(currentMethod == MULTI_SEARCH_METHOD_NO) {
					toggleDirection2 ();
				}
			}
	      break;

		case Qt::Key_Right:
			if(!keyRightEvent())
				return QWidget::eventFilter (o, e);
			break;

		case Qt::Key_Left:
			if(!keyLeftEvent())
				return QWidget::eventFilter (o, e);
			break;

	    case Qt::Key_Delete:
			if(currentMethod == MAIN_SEARCH_METHOD_NO ||
			   currentMethod == CONDITION_SEARCH_METHOD_NO ||
			   currentMethod == COMBINATION_SEARCH_METHOD_NO )
				clearLine ();
			break;

		case Qt::Key_Home:
		case Qt::Key_BackSpace:
			emit checkBackNavi();
			if(isNaviEnable)
				back();
			break;
		case Qt::Key_End:
			emit checkForwardNavi();
			if(isNaviEnable)
				forward();
			break;
		case Qt::Key_PageDown:
			if(isBookMarkShown) {
				showDlgWait();
				break;
			}
			down();
			break;

		case Qt::Key_Return:
			if(!hasFocusLineEdit) {
				QString text = getSelectedText();
				if(!text.isEmpty()) {
					lookup(text);
					break;
				}
			}

			if(currentMethod == MAIN_SEARCH_METHOD_NO ||
			   currentMethod == CONDITION_SEARCH_METHOD_NO ||
			   currentMethod == COMBINATION_SEARCH_METHOD_NO ) {
				if( (displayMode ==DISP_LIST) ||
					(displayMode =DISP_LIST_AND_CONTENT) ) {
					displayMode =DISP_CONTENT;
					changeDispContent(displayMode);
				}
			}
			else if(currentMethod == MULTI_SEARCH_METHOD_NO) {
				if( (displayMode2 ==DISP_LIST) ||
					(displayMode2 =DISP_LIST_AND_CONTENT) ) {
					displayMode2 =DISP_CONTENT;
					changeDispContent2(displayMode2);
				}
			}
			break;

	    default:
			return FALSE;
	    }

		return TRUE;	  
	}

	return QWidget::eventFilter (o, e);

}

bool Zten::keyRightEvent()
{

	if(currentMethod == MAIN_SEARCH_METHOD_NO ||
	   currentMethod == CONDITION_SEARCH_METHOD_NO ||
	   currentMethod == COMBINATION_SEARCH_METHOD_NO ) {
		if(displayMode ==DISP_LIST_AND_CONTENT) {
			if(EntryList->hasFocus())
				ContentBrowser->setFocus ();
		}
		else if( displayMode ==DISP_CONTENT)
			return FALSE;
	}
	else if(currentMethod == MULTI_SEARCH_METHOD_NO) {
		if(displayMode2 ==DISP_LIST_AND_CONTENT) {
			if(EntryList2->hasFocus())
				ContentBrowser2->setFocus ();
		}
		else if( displayMode2 ==DISP_CONTENT)
			return FALSE;

	}
	return TRUE;
}

bool Zten::keyLeftEvent()
{

	if(currentMethod == MAIN_SEARCH_METHOD_NO ||
	   currentMethod == CONDITION_SEARCH_METHOD_NO ||
	   currentMethod == COMBINATION_SEARCH_METHOD_NO ) {
		if(displayMode ==DISP_LIST_AND_CONTENT) {
			if(ContentBrowser->hasFocus())
				EntryList->setFocus ();
		}
		else if( displayMode ==DISP_CONTENT)
			return FALSE;
	}
	else if(currentMethod == MULTI_SEARCH_METHOD_NO) {
		if(displayMode2 ==DISP_LIST_AND_CONTENT) {
			if(ContentBrowser2->hasFocus())
				EntryList2->setFocus ();
		}
		else if( displayMode2 ==DISP_CONTENT)
			return FALSE;

	}
	return TRUE;
}


bool Zten::keyEscapeEvent()
{

	if(currentMethod == MAIN_SEARCH_METHOD_NO ||
	   currentMethod == CONDITION_SEARCH_METHOD_NO ||
	   currentMethod == COMBINATION_SEARCH_METHOD_NO ) {

		if(displayMode == DISP_CONTENT) {
			displayMode =DISP_LIST_AND_CONTENT;
			changeDispContent(displayMode);
			return FALSE;
		}
	}
	else if(currentMethod == MULTI_SEARCH_METHOD_NO) {
		if(displayMode2 == DISP_CONTENT) {
			displayMode2 =DISP_LIST_AND_CONTENT;
			changeDispContent2(displayMode2);
			return FALSE;
		}
	}

	return TRUE;
}

void
Zten::setImageScale (int scale)
{
  QBookEntry *be;
  cur_scale = scale;

  for (be = books.first (); be != NULL; be = books.next ())
    {
      be->setImageScale (scales[scale+2]);
    }

	if(isBookMarkShown) {
		bookmark_search(&current_bk);
		return;
	}

  if (history_current >= 0)
    goToNoHistory (history[history_current]);
}
  
void
Zten::setCurrentFont (QFont font)
{
  cur_font = font;
  EntryList->setFont (font);
  EntryList->resizeContents(EntryList->visibleWidth(),EntryList->visibleHeight());
  EntryList2->setFont (font);
  EntryList2->resizeContents(EntryList2->visibleWidth(),EntryList2->visibleHeight());
  ContentBrowser->setFont (font);
  ContentBrowser->resizeContents(ContentBrowser->visibleWidth(),ContentBrowser->visibleHeight());
  ContentBrowser2->setFont (font);
  ContentBrowser2->resizeContents(ContentBrowser2->visibleWidth(),ContentBrowser2->visibleHeight());
  ContentBrowser3->setFont (font);
  ContentBrowser3->resizeContents(ContentBrowser3->visibleWidth(),ContentBrowser3->visibleHeight());
  QBookEntry *be;
  
  for (be = books.first (); be != NULL; be = books.next ())
    {
      be->setFontSize (font.pixelSize ());
    }

	if(isBookMarkShown) {
		bookmark_search(&current_bk);
		return;
	}

  if (history_current >= 0)
    goToNoHistory (history[history_current]);
}

void
Zten::setCurrentFont (int id)
{
	cur_font_id = id;

	QFont f = font();
	f.setPointSize(SlMisc::getFontPointSize(cur_font_id));

	cur_font = f;
	EntryList->setFont (cur_font);
	EntryList->resizeContents(EntryList->visibleWidth(),EntryList->visibleHeight());
	EntryList2->setFont (cur_font);
	EntryList2->resizeContents(EntryList2->visibleWidth(),EntryList2->visibleHeight());
	ContentBrowser->setFont (cur_font);
	ContentBrowser->resizeContents(ContentBrowser->visibleWidth(),ContentBrowser->visibleHeight());
	ContentBrowser2->setFont (cur_font);
	ContentBrowser2->resizeContents(ContentBrowser2->visibleWidth(),ContentBrowser2->visibleHeight());
	ContentBrowser3->setFont (cur_font);
	ContentBrowser3->resizeContents(ContentBrowser3->visibleWidth(),ContentBrowser3->visibleHeight());
	QBookEntry *be;
  
	for (be = books.first (); be != NULL; be = books.next ())
    {
		be->setFontSize (f.pixelSize ());
    }

	if(isBookMarkShown) {
		bookmark_search(&current_bk);
		return;
	}

	if (history_current >= 0) {
		goToNoHistory (history[history_current]);
	}

}


void
Zten::setDictionary ()
{
	ZtenSetDict *setDict = new ZtenSetDict(this, "SetDict", true);

	setDict->showMaximized ();
	setDict->setDictList(&dictList);
	setDict->exec ();  

	if (setDict->result() == QDialog::Accepted) {
		setDict->checkAvailableDict(&dictList);
		if(!setDict->isListChanged()) {
			delete setDict;
			return;
		}

		closeBooks();
		dictList.clear();
		setDict->updateDictList(&dictList);

		openBooks();
		initBookMark();

		if(currentMethod == MULTI_SEARCH_METHOD_NO)
			backToMultiCond();
		if( (currentMethod == MULTI_SEARCH_METHOD_NO) ||
			(currentMethod == MENU_SEARCH_METHOD_NO) )
			changeLayout(currentMethod);

		isDictChanged = TRUE;

		if(isBookMarkShown) {
			checkBookMarkDict();
		}
		else {
			if( (currentMethod == MAIN_SEARCH_METHOD_NO) ||
				(currentMethod == CONDITION_SEARCH_METHOD_NO) ||
				(currentMethod == COMBINATION_SEARCH_METHOD_NO) )
				activateAndSearch();
		}

		writeConfig();
	}

	delete setDict;
}

void
Zten::reload ()
{
	//closeBooks ();
	//changeMainSearchLayout(TRUE);
}

bool
Zten::menu_goToNoHistory (EB_Position pos, bool isWaitDisp)
{
	if(isWaitDisp) {
		SlDlgWait dlg(this,tr("Please wait..."));
		dlg.exec();
	}

	QString str = current_book->getTextPatch (pos, TRUE);
	if(str.isEmpty())
		return FALSE;
	current_browser->setText(str);
	return TRUE;
}

void
Zten::menu_goTo (EB_Position pos, bool isWaitDisp)
{
	current_pos = pos;
	if(!menu_goToNoHistory (pos, isWaitDisp))
		return;

	if (history_current == (int) history.size() - 1) {
		history.resize(history.size() + 1);
    }
	history[++history_current] = pos;
	if (history.size () != 1) emit makeHistory ();
}

void
Zten::linkJump3 (EB_Position pos)
{

	SlDlgWait dlg(this,tr("Please wait..."));
	dlg.exec();

	menu_goTo(pos, FALSE);
}

void
Zten::linkJump_menu (int pos)
{

	SlDlgWait dlg(this,tr("Please wait..."));
	dlg.exec();

	EB_Position p = current_book->getLinkPosition (pos);
	if (p.page >= 0) menu_goTo (p, FALSE);
}

bool
Zten::redisplay (SlListBoxItem *item)
{
	if(isBookMarkShown)
		return TRUE;

	if (item)
    {
		QBookHitItem *bi = (QBookHitItem *)item;
		if (bi == NULL)
			return FALSE;
		QBookEntry *be;
		be = bi->getBookEntry ();
		if (be)
		{
			history.resize (0);
			history_current = -1;
			emit noHistory ();
			current_book = be;
			current_headpos = be->getHitHeadPosition(bi->getHitPos());

			goTo(be->getHitPosition(bi->getHitPos ()));

			return TRUE;
		}
    }
	return FALSE;
}

void
Zten::itemSelected (SlListBoxItem *item)
{
	if(isSearching)
		return;


	if(item) {
		backCount = 1;

		EntryList->setSelectionMode(SlListBox::Single);
		if( displayMode == DISP_LIST_AND_CONTENT) {
			redisplay (item);
			current_bk_pos = current_pos;
		}
		else {
			setCurrentBkPos(item);
		}
   }
}

void
Zten::itemSelected2 (SlListBoxItem *item)
{
	if(isSearching)
		return;

	if (item) {
		backCount = 1;

		EntryList2->setSelectionMode(SlListBox::Single);
		if(displayMode2 == DISP_LIST_AND_CONTENT) {
			redisplay (item);
			current_bk_pos = current_pos;
		}
		else {
			setCurrentBkPos(item);
		}
    }
}

void
Zten::lookup (QString key)
{
	if(currentMethod != MAIN_SEARCH_METHOD_NO) {
		searchMethodChanged(MAIN_SEARCH_METHOD_NO);
		setCurrentMethod(MAIN_SEARCH_METHOD_NO);
		searchTypeSelected(SEARCH_HEAD);
	}
	else {
		searchTypeSelected(SEARCH_HEAD);
	}

	LineEdit->setEditText (key);
	LineEdit->activate ();
	normalsearch ();
}

void
Zten::setIncremental (bool on)
{
  incremental_mode = on;
  emit incrementalIsSet (on);
}


void
Zten::setContinuous (bool on)
{
  continuous_mode = on;
  emit continuousIsSet (on);
}

void
Zten::focusInLineEdit(bool b)
{

	hasFocusLineEdit = b;

}

QString Zten::getSelectedText()
{
	if(!ContentBrowser->isHidden() &&
	   ContentBrowser->hasSelectedText()) {
		return ContentBrowser->selectedText();
	}
	else if(!ContentBrowser2->isHidden() &&
	   ContentBrowser2->hasSelectedText()) {
		return ContentBrowser2->selectedText();
	}
	else if(!ContentBrowser3->isHidden() &&
	   ContentBrowser3->hasSelectedText()) {
		return ContentBrowser3->selectedText();
	}

	return QString("");
}

void
Zten::activateAndSearch()
{
	backCount = 1;
	isSearching = true;

	if( !hasFocusLineEdit) {
		QString text = getSelectedText();
		if(!text.isEmpty()) {
			isBookMarkShown = FALSE;
			lookup(text);
			isSearching = false;
			return;
		}
	}

	if(isBookMarkShown) {
		if(LineEdit->currentText().isEmpty())
			return;
	}
	isBookMarkShown = FALSE;


	switch(currentMethod) {
	case MAIN_SEARCH_METHOD_NO:
		LineEdit->activate ();
		normalsearch ();
		break;
	case MULTI_SEARCH_METHOD_NO:
		multiSearch();
		break;
	case CONDITION_SEARCH_METHOD_NO:
		LineEdit->activate ();
		normalsearch();
		break;
	case COMBINATION_SEARCH_METHOD_NO:
		LineEdit->activate ();
		combinationSearch();
		break;
	default:
		break;
	}

	isSearching = false;

}

void
Zten::clearLine ()
{
	clearField();

	isBookMarkShown = FALSE;

	displayMode =DISP_LIST;
	changeDispContent(DISP_LIST);

	emit updateBookMarkAction(FALSE);
}

void
Zten::clearField ()
{
	LineEdit->setFocus ();
	LineEdit->setEditText ("");

	current_entrylist->clear();
	current_browser->setText("");
}


QString Zten::convertChuten(QString str)
{

	return str.replace(tr("ZenkakuChuten"), tr("HankakuChuten"));

}

int Zten::search_normal(QBookEntry *be, QString word, int type)
{
	int matches = be->search (word,type);

	for (int i = 0 ; i < matches; i ++)	{
		QString heading = be->getHeading (i);
		if (!heading.isNull())  
			EntryList->insertItem (new QBookHitItem
								   (convertChuten(heading) +
									" [" + be->getTitle () + "]",
									be, i), -1);
	}
	return matches;
}

int Zten::search_combination(QBookEntry *be, QString word, int type, int counter)
{
	if(counter > MAX_HITS_BASE)
		return 0;

	int listCount = EntryList->count();
	int matches = be->search_combi (word,counter,type);
	for (int i = 0 ; i < matches; i ++)	{
		QString heading = be->getHeading (i+counter);
		if (!heading.isNull()) {
			QString entryStr = convertChuten(heading) +
				" [" + be->getTitle() + "]";
			if(listCount == 0) {
				EntryList->insertItem (new QBookHitItem(entryStr, be, i), -1);
			}
			else {
				bool isExist = FALSE;
				for(int j=0; j< listCount; j++) {
					if(entryStr == EntryList->text(j)) {
						isExist = TRUE;
						break;
					}
				}
				if(!isExist) {
					QBookHitItem* newStr = new QBookHitItem(entryStr, be, i+counter);
					EntryList->insertItem (newStr, -1);
				}
			}
		}
	}
	return matches;
}

void
Zten::normalsearch ()
{
	QBookEntry *be;
	int total_matches = 0;

	QString word = expectIrregularWord(LineEdit->currentText());

	if(word.isEmpty())
		return;

	EntryList->clear();
	if (word.length () >= 1) {
		for (be = books.first (); be != NULL; be = books.next ()) {
			// search type equals to SearchTypeComboBox->currentItem ();

			int type;
			if(currentMethod == MAIN_SEARCH_METHOD_NO) {
				type = getSearchType();
			}
			else  { //if (currentMethod == CONDITION_SEARCH_METHOD_NO) {
				type = KEYWORD_MATCH;
			}

			total_matches += search_normal(be, word, type);
		}
    }

	updateEntryList1(total_matches);

	if (total_matches != 0) {
		displayMode = DISP_LIST;

		if(isDictChanged)
			EntryList->setSelected(0, true);

		toggleContentForce (DISP_LIST_AND_CONTENT);
		current_bk_pos = current_pos;

		if(isDictChanged) {
			isDictChanged = FALSE;
		}

		EntryList->setSelected(0, true);

    }
	else {
		toggleContentForce (DISP_LIST);
		LineEdit->setFocus();
		emit dispContentChanged(displayMode);
	}
}

void
Zten::searchIncremental (const QString & word)
{
	QString sword = expectIrregularWord(word);
	int total_matches = 0;

	if (incremental_mode && currentMethod == MAIN_SEARCH_METHOD_NO) {
		backCount = 1;
		toggleContentForce (DISP_LIST);
		EntryList->clear();
		if (sword.length () >= 1) {
			QBookEntry *be;
			for (be = books.first (); be != NULL; be = books.next ()) {
				total_matches += search_normal(be, sword, getSearchType());
			}
			EntryList->setCurrentItem(0);

			if(total_matches != 0) {
				emit updateBookMarkAction(TRUE);
			}
			else {
				emit updateBookMarkAction(FALSE);
			}
			
			emit updateDownAction(FALSE);

		}
		else {
			ContentBrowser->setText ("");
			emit updateDownAction(FALSE);
			emit updateBookMarkAction(FALSE);
		}
    }
}

void
Zten::combinationSearch ()
{
	if(currentMethod != COMBINATION_SEARCH_METHOD_NO)
		return;

	QBookEntry *be;
	int total_matches = 0;
	QString word = expectIrregularWord(LineEdit->currentText());

	EntryList->clear();
	if (word.length () >= 1) {
		for (be = books.first (); be != NULL; be = books.next ()) {
			// search type equals to SearchTypeComboBox->currentItem ();
			int dict_matches = 0;

			if(SearchTypePopup->isItemChecked(SEARCH_HEAD)) {
				dict_matches += search_combination(be, word, SEARCH_HEAD-1,dict_matches);
			}
			if(SearchTypePopup->isItemChecked(SEARCH_TAIL)) {
				dict_matches += search_combination(be, word, SEARCH_TAIL-1,dict_matches);
			}
			if(SearchTypePopup->isItemChecked(SEARCH_COND)) {
				dict_matches += search_combination(be, word, KEYWORD_MATCH,dict_matches);
			}
			total_matches += dict_matches;

		}
    }

	updateEntryList1(total_matches);

	if (total_matches != 0) {
		displayMode = DISP_LIST;

		if(isDictChanged)
			EntryList->setSelected(0, true);

		toggleContentForce (DISP_LIST_AND_CONTENT);
		current_bk_pos = current_pos;

		if(isDictChanged)
			isDictChanged = FALSE;

		EntryList->setSelected(0, true);

	}
	else {
		toggleContentForce (DISP_LIST);
		LineEdit->setFocus();
		emit dispContentChanged(displayMode);
	}

}

void
Zten::addBookmark()
{
	QString entry = current_entrylist->currentText();
	if((current_entrylist->count() == 0) || entry.isEmpty() )
		return;

	bookmark = new BookMark();

	bookmark->setPosition(current_bk_pos);
	bookmark->setHeadPosition(current_headpos);
	bookmark->setBookTitle(current_book->getTitle());

	int pSize = 20;
	QString sizeStr, sizeStr2;
	sizeStr.setNum(pSize);
	sizeStr2.setNum(pSize);
	int index, pos1, pos2, pos3;
	index = pos1 = pos2 = pos3 = 0;

	while ((pos1 = entry.find ("<img src=", index)) >= 0) {
		if( (pos2 = entry.find("width=",pos1)) == -1) {
			index++;
			continue;
		}

		if( (pos3 = entry.find("height=",pos2)) == -1) {
			index++;
			continue;
		}
		QString wStr = entry.mid(pos2+6, 2);
		int w = wStr.toInt();
		QString hStr = entry.mid(pos3+7, 2);
		int h = hStr.toInt();

		index = pos2;
		if( pSize == h )
			continue;

		if(w*2 == h)
			sizeStr2.setNum(pSize/2);
		else
			sizeStr2.setNum(pSize);

		int len = strlen("width=20 height=20");
		if(w<10)
			len--;

		QString repStr = "width=" + sizeStr2 +" height=" + sizeStr;
		entry = entry.replace(pos2, len, repStr);

	}

	bookmark->setEntry(entry);

	bool sameExist = FALSE;
	BookMark *bk;
	for (bk = markList.first(); bk != 0; bk = markList.next()){
		if(entry == bk->getEntry()) {
			markList.remove(bk);
			sameExist = TRUE;
			break;
		}
	}

	if(!sameExist) {
		if(markList.count() >= MARKLIST_MAX_NUM) {
			bk = markList.first();
			markList.remove(bk);
		}
	}

	SlDlgWait* dlgwait;
	int scale = SlMisc::getResolutionScale();

	pos1 = entry.findRev("[",-1);
	if(pos1 > 2)
		pos1--;
		
	QString tmp = entry.mid(0, pos1);
	
	tmp.replace(QString("<sup>"),QString(""));
	tmp.replace(QString("</sup>"),QString(""));
	tmp.replace(QString("<small>"),QString(""));
	tmp.replace(QString("</small>"),QString(""));

	int max_len = BOOKMARK_DISP_MAX_LEN;
	pos1 = 0;
	for(int i=0; i<BOOKMARK_DISP_MAX_LEN; i++) {

		pos1 = tmp.find("<img",pos1);

		if(pos1 > 0)
			pos2 = tmp.find(">",pos1);
		if( pos2 > pos1) {
			max_len += (pos2-pos1);
		}
		
	}
	if(tmp.length() > (uint)max_len) {
		tmp.truncate(max_len);
		tmp += "...";
	}

	QString str = tr("[") + tmp + tr("]") + "\n" + tr("add BookMark");

	dlgwait = new SlDlgWait(0, str);
	dlgwait->resize(230*scale,60*scale);
	dlgwait->exec();

	markList.append(bookmark);

	usleep(10*100000);

	writeConfig();

	if(dlgwait)
		delete dlgwait;
	dlgwait = 0;
}

void Zten::initBookMark()
{
	
	QBookEntry *be;
	BookMark *bk;
	for (bk = markList.first(); bk != 0; bk = markList.next()){
		for (be = books.first (); be != NULL; be = books.next ()) {
			if(be->getTitle() == bk->getBookTitle()) {
				EB_Position head_pos = bk->getHeadPosition();
				QString entryStr = be->getHeading(head_pos);
				break;
			}
		}
	}
}

void Zten::openBookmark()
{
	ZtenBookMark *bkListView = new ZtenBookMark(this, "bkListView", true);

	bkListView->showMaximized();
	bkListView->setBookMarkList(&markList);
	bkListView->exec();  

	markList.clear();
	bkListView->updateBookMarkList(&markList);

    if (bkListView->result() == QDialog::Accepted) {

		QString selectEntry= bkListView->getCurrentEntry();

		if(!selectEntry.isEmpty()) {

			BookMark *bk;
			for (bk = markList.first(); bk != 0; bk = markList.next()){
				if(selectEntry == bk->getEntry())
					break;
			}

			searchMethodChanged(MAIN_SEARCH_METHOD_NO);
			setCurrentMethod(MAIN_SEARCH_METHOD_NO);
			searchTypeSelected(SEARCH_HEAD);

			displayMode = DISP_LIST_AND_CONTENT;

			isBookMarkShown = FALSE;
			bookmark_search(bk);

		}
	}

	delete bkListView;
}

void Zten::bookmark_search(BookMark *bk)
{
	QBookEntry *be;

	QString entry = bk->getEntry();
	EB_Position pos = bk->getPosition();
	EB_Position head_pos = bk->getHeadPosition();
	QString title = bk->getBookTitle();

	bool isBookMatch = FALSE;
	for (be = books.first (); be != NULL; be = books.next ()) {
		if(be->getTitle() == title) {
			isBookMatch = TRUE;
			break;
		}
	}
	if(!isBookMatch) {
		QMessageBox::warning(this, QObject::tr("Error"),
				 tr("Bookmark page cannot be opened because\nthere is no dictionary selected."));
		return;
	}


	SlDlgWait dlg(this,tr("Please wait..."));
	dlg.exec();

	current_book = be;

	LineEdit->setEditText ("");

	EntryList->clear();

	QString entryStr = be->getHeading(head_pos);
	entryStr = entryStr + " [" + title + "]";
	EntryList->insertItem (new QBookHitItem(entryStr, be, 0), -1);

	isBookMarkShown = TRUE;
	updateEntryList1(1);
	changeDispContent(displayMode);
	ContentBrowser->setText (be->getText (pos, FALSE));

	if(displayMode != DISP_LIST)
		ContentBrowser->show ();
	EntryList->setSelected(0, true);
	EntryList->setFocus();

	if(displayMode == DISP_CONTENT)
		ContentBrowser->setFocus();

	emit updateDownAction(FALSE);
	emit updateBookMarkAction(FALSE);

	setCurrentBookMark(bk);

}

void Zten::setCurrentBookMark(BookMark *bk)
{
	current_bk.setEntry(bk->getEntry());
	current_bk.setBookTitle(bk->getBookTitle());
	current_bk.setPosition(bk->getPosition());
	current_bk.setHeadPosition(bk->getHeadPosition());

}

void Zten::multiSearchTextChanged(const QString& text, int id)
{
	if(isSetCandidate) {
		isSetCandidate = false;
		return;
	}

	if(text.isEmpty() && (id == 2)) {
		if(isGaijiDisp) {
			isGaijiDisp = FALSE;
			MultiLabel6->hide();
			gaijiBrowser->hide();
		}
	}

	if(!isGaijiDisp) {
		if(id == 1)
			candidate1 = "";
		else if(id == 2)
			candidate2 = "";
		else if(id == 3)
			candidate3 = "";
		else if(id == 4)
			candidate4 = "";
		else if(id == 5)
			candidate5 = "";
	}

}

void Zten::slotTextChanged1(const QString& text)
{
	multiSearchTextChanged(text, 1);
}

void Zten::slotTextChanged2(const QString& text)
{
	multiSearchTextChanged(text, 2);
}

void Zten::slotTextChanged3(const QString& text)
{
	multiSearchTextChanged(text, 3);
}

void Zten::slotTextChanged4(const QString& text)
{
	multiSearchTextChanged(text, 4);
}

void Zten::slotTextChanged5(const QString& text)
{
	multiSearchTextChanged(text, 5);
}

QString Zten::checkGaijiStr(QString text)
{

	QString ret = "";

	int pos = text.find("<img src=");
	if(pos > 0)
		return ret;
	pos = text.find("GAIJI(");
	if(pos <= 0)
		return ret;

	QString c1,c2;
	c1 = text.mid(pos+6,2);
	c2 = text.mid(pos+8,2);

	char c[3];
	bool ok;
	c[0] = c1.toInt(&ok,16);
	c[1] = c2.toInt(&ok,16);
	c[2] = 0x00;
	ret = c;
	return ret;

}

void
Zten::setCandidateLabel (int index, QString text, QString label, bool gaiji)
{
	QString ltext = "";
	QString gaijistr;

	isSetCandidate = true;

	if(gaiji) {
		gaijistr = checkGaijiStr(text);
		if(!gaijistr.isEmpty()) {
			isGaijiDisp = TRUE;
			ltext = text;
			text = gaijistr;
			if(!label.isEmpty())
				text += label;

			label = tr("The following reference");

			gaijiBrowser->setText(ltext);
			MultiLabel6->show();
			gaijiBrowser->show();
		}
	}
	else {
		if(index == 2) {
		gaijiBrowser->setText("");
		MultiLabel6->hide();
		gaijiBrowser->hide();
		}
	}

  switch (index)
    {
    case 1:
      candidate1 = QString (text);
      LineEdit1->setText (label);
      break;
    case 2:
      candidate2 = QString (text);
      LineEdit2->setText (label);
      break;
    case 3:
      candidate3 = QString (text);
      LineEdit3->setText (label);
      break;
    case 4:
      candidate4 = QString (text);
      LineEdit4->setText (label);
      break;
    case 5:
      candidate5 = QString (text);
      LineEdit5->setText (label);
      break;
    }
}

void
Zten::multiCandidate1 ()
{
  QBookMultiSearch *ms = ms_list.at (MultiSearchCombo->currentItem ());
  EB_Position pos;
  EB_Book *book = ms->getBookEntry ()->getBook ();
  if (eb_multi_entry_candidates (book,
				 ms->getMultiId (),
				 0,
				 &pos) == EB_SUCCESS)
    {
      ZtenCandidate c (book, pos, 1, this, 0, true);
      connect (&c, SIGNAL (candidateDecided (int, QString, QString, bool)),
	       this, SLOT (setCandidateLabel (int, QString, QString, bool)));
      c.showMaximized ();
      c.exec ();  
    }
  LineEdit1->setFocus();
}

void
Zten::multiCandidate2 ()
{
  QBookMultiSearch *ms = ms_list.at (MultiSearchCombo->currentItem ());
  EB_Position pos;
  EB_Book *book = ms->getBookEntry ()->getBook ();
  if (eb_multi_entry_candidates (book,
				 ms->getMultiId (),
				 1,
				 &pos) == EB_SUCCESS)
    {
      ZtenCandidate c (book, pos, 2, this, 0, true);
      connect (&c, SIGNAL (candidateDecided (int, QString, QString, bool)),
	       this, SLOT (setCandidateLabel (int, QString, QString, bool)));
      c.showMaximized ();
      c.exec ();  
    }
  LineEdit2->setFocus();
}

void
Zten::multiCandidate3 ()
{
  QBookMultiSearch *ms = ms_list.at (MultiSearchCombo->currentItem ());
  EB_Position pos;
  EB_Book *book = ms->getBookEntry ()->getBook ();
  if (eb_multi_entry_candidates (book,
				 ms->getMultiId (),
				 2,
				 &pos) == EB_SUCCESS)
    {
      ZtenCandidate c (book, pos, 3, this, 0, true);
      connect (&c, SIGNAL (candidateDecided (int, QString, QString, bool)),
	       this, SLOT (setCandidateLabel (int, QString, QString, bool)));
      c.showMaximized ();
      c.exec ();  
    }
  LineEdit3->setFocus();
}

void
Zten::multiCandidate4 ()
{
  QBookMultiSearch *ms = ms_list.at (MultiSearchCombo->currentItem ());
  EB_Position pos;
  EB_Book *book = ms->getBookEntry ()->getBook ();
  if (eb_multi_entry_candidates (book,
				 ms->getMultiId (),
				 3,
				 &pos) == EB_SUCCESS)
    {
      ZtenCandidate c (book, pos, 4, this, 0, true);
      connect (&c, SIGNAL (candidateDecided (int, QString, QString, bool)),
	       this, SLOT (setCandidateLabel (int, QString, QString, bool)));
      c.showMaximized ();
      c.exec ();  
    }
  LineEdit4->setFocus();
}

void
Zten::multiCandidate5 ()
{
  QBookMultiSearch *ms = ms_list.at (MultiSearchCombo->currentItem ());
  EB_Position pos;
  EB_Book *book = ms->getBookEntry ()->getBook ();
  if (eb_multi_entry_candidates (book,
				 ms->getMultiId (),
				 4,
				 &pos) == EB_SUCCESS)
    {
      ZtenCandidate c (book, pos, 5, this, 0, true);
      connect (&c, SIGNAL (candidateDecided (int, QString, QString, bool)),
	       this, SLOT (setCandidateLabel (int, QString, QString, bool)));      
      c.showMaximized ();
      c.exec ();  
    }
  LineEdit5->setFocus();
}

void
Zten::playSound (int index)
{
  SlDlgWait dlg(this,tr("Please wait..."));
  dlg.exec();
  if (current_book->playSound (index)!=0) {
    QMessageBox::warning(this,QObject::tr("Error"),
			 tr("Unable to open audio device"));
  }
}

void
Zten::playMpeg (int index)
{
	if(playMpegFlag)
		return;
	if(current_book->playMpeg(index))
		playMpegFlag = TRUE;
	else {
		playMpegFlag = FALSE;

		QMessageBox::warning(this,QObject::tr("Error"),
					 tr("Unable to play this moive"));
	}
	QTimer::singleShot(5*1000, this, SLOT(slotClearMpegFlag())); // forbid to launch Movie Player for 5 sec.
}

void
Zten::setPlayMpegFlag(bool b)
{
	playMpegFlag = b;

}

bool
Zten::backAction ()
{
	if(isBookMarkShown)
		return FALSE;

	if(currentMethod == MENU_SEARCH_METHOD_NO)
		return FALSE;

	if(backCount > BACK_ACTION_MAX_COUNT)
		return FALSE;

	SlDlgWait dlg(this,tr("Please wait..."));
	dlg.exec();

	QString str = current_book->getBackText (current_pos, backCount);

	if(str.isEmpty())
		return FALSE;

	backCount++;
	current_browser->setText(str);

	return TRUE;

}

bool
Zten::goToNoHistory (EB_Position pos)
{
	QString str;

	SlDlgWait dlg(this,tr("Please wait..."));
	dlg.exec();

	if(currentMethod == MENU_SEARCH_METHOD_NO)
		str = current_book->getText (pos, TRUE);
	else
		str = current_book->getText (pos, continuous_mode);

	if(str.isEmpty())
		return FALSE;

	current_browser->setText(str);
	return TRUE;

}

void
Zten::goTo (EB_Position pos)
{
	if(isBookMarkShown) {
		showDlgWait();
		return;
	}

	current_pos = pos;
	if(!goToNoHistory (pos))
		return;

	if (history_current == (int) history.size() - 1) {
		history.resize(history.size() + 1);
    }
	history[++history_current] = pos;
	if (history.size () != 1) emit makeHistory ();
}

void
Zten::back ()
{
  emit followHistory ();
  if (history_current >= 1)
    {
      history_current--;
      EB_Position p = history[history_current];
	  if(currentMethod == MENU_SEARCH_METHOD_NO)
		  menu_goToNoHistory (p, TRUE);
	  else
		  goToNoHistory (p);
    }
  if (history_current == 0)
    {
      emit beginningOfHistory ();
    }
}

void
Zten::forward ()
{
  emit followHistory ();
  if (history_current < (int) history.size() - 1)
    {
      history_current++;
      EB_Position p = history[history_current];
	  if(currentMethod == MENU_SEARCH_METHOD_NO)
		  menu_goToNoHistory (p, TRUE);
	  else
		  goToNoHistory (p);
    }
  if (history_current == (int) history.size() - 1)
    {
      emit endOfHistory ();
    }
}

void
Zten::down ()
{
  if (current_book)
    {
      EB_Position p;
      if (eb_tell_text(current_book->getBook(), &p) != EB_SUCCESS)
		  return;
      int a = p.page * 2048 + p.offset ;
      p.page = a/2048;
      p.offset = a%2048;
	  if(currentMethod == MENU_SEARCH_METHOD_NO)
		  menu_goTo(p, TRUE);
	  else
		  goTo(p);
    }
}


void
Zten::gotoMenu (QBookEntry *be)
{

  EB_Position p;
  if (eb_menu(be->getBook(), &p) != EB_SUCCESS) {
    return;
  }
  history.resize (0);
  history_current = -1;
  emit noHistory ();
  current_browser = ContentBrowser3;
  goTo (p);
}

void
Zten::multiSelected (int index)
{
  QBookEntry *be = NULL;
  QBookMultiSearch *ms = NULL;
  multi_number = 0;
  
  if (ms_list.count () > 0)
    {
      MultiSearchCombo->setEnabled (TRUE);
      ms = ms_list.at (index);
      be = ms->getBookEntry ();
    }

  // FIXME: Hard coding.
  MultiLabel1->setText ("");
  LineEdit1->setEnabled (FALSE);
  LineEdit1->setText ("");
  MultiCandidate1->setEnabled (FALSE);
  if (!candidate1.isNull ())
    candidate1 = QString ();

  MultiLabel2->setText ("");
  LineEdit2->setEnabled (FALSE);
  LineEdit2->setText ("");
  MultiCandidate2->setEnabled (FALSE);
  if (!candidate2.isNull ())
    candidate2 = QString ();
  
  MultiLabel3->setText ("");
  LineEdit3->setEnabled (FALSE);
  LineEdit3->setText ("");
  MultiCandidate3->setEnabled (FALSE);
  if (!candidate3.isNull ())
    candidate3 = QString ();
  
  MultiLabel4->setText ("");
  LineEdit4->setEnabled (FALSE);
  LineEdit4->setText ("");
  MultiCandidate4->setEnabled (FALSE);
  if (!candidate4.isNull ())
    candidate4 = QString ();
  
  MultiLabel5->setText ("");
  LineEdit5->setEnabled (FALSE);
  LineEdit5->setText ("");
  MultiCandidate5->setEnabled (FALSE);
  if (!candidate5.isNull ())
    candidate5 = QString ();

  if (be && ms && ms_list.count () > 0)
    {
      be->findMultiSearchEntry (ms->getMultiId ());
      multi_last_book_ident = be->getIdentity ();
      multi_last_id = ms->getMultiId ();
    }
}

void
Zten::multiSearch ()
{
	SlDlgWait dlg(this,tr("Please wait..."));
	dlg.exec();

	// FIXME: qbookentry interface should have EUC API.
	QTextCodec *codec = QTextCodec::codecForName("eucJP");
	if (current_book)  {
		QBookMultiSearch *ms = ms_list.at (MultiSearchCombo->currentItem ());
		QList<QString> multi_words;
		multi_words.setAutoDelete (TRUE);

		if(LineEdit1->text().isEmpty())
			candidate1 = "";
		if(LineEdit2->text().isEmpty())
			candidate2 = "";
		if(LineEdit3->text().isEmpty())
			candidate3 = "";
		if(LineEdit4->text().isEmpty())
			candidate4 = "";
		if(LineEdit5->text().isEmpty())
			candidate5 = "";

		if (multi_number >= 1)
			multi_words.append(new QString (candidate1.isEmpty ()?
					codec->fromUnicode (LineEdit1->text ()) :
					candidate1));
		if (multi_number >= 2)
			multi_words.append(new QString (candidate2.isEmpty ()?
					codec->fromUnicode (LineEdit2->text ()) :
					candidate2));
		if (multi_number >= 3)
			multi_words.append(new QString (candidate3.isEmpty ()?
					codec->fromUnicode (LineEdit3->text ()) :
					candidate3));
		if (multi_number >= 4)
			multi_words.append(new QString (candidate4.isEmpty ()?
					codec->fromUnicode (LineEdit4->text ()) :
					candidate4));	
		if (multi_number == 5)
			multi_words.append(new QString (candidate5.isEmpty ()?
					codec->fromUnicode (LineEdit5->text ()) :
					candidate5));		

		int matches = current_book->multiSearch (ms->getMultiId (), multi_words);

		EntryList2->clear ();
		for (int i = 0; i < matches; i ++) {
			QString heading = current_book->getHeading (i);
			if (!heading.isNull()) {
				EntryList2->insertItem (new QBookHitItem
				      (convertChuten(heading) + " [" +
				       current_book->getTitle () + "]",
				       current_book, i), -1);	  
			}
		}
		int count = 100;
		int next_matches = 0;
		while((next_matches = current_book->getNextHitList()) > 0) {
			for (int i = 0; i < next_matches; i ++) {
				QString heading = current_book->getHeading (i+count);
				if (!heading.isNull()) {
					EntryList2->insertItem (new QBookHitItem
				      (convertChuten(heading) + " [" +
				       current_book->getTitle () + "]",
				       current_book, i+count), -1);	  
				}
			}
			count+=next_matches;
			matches += next_matches;

			if(count>MAX_MULTI_HITS_BASE)
				break;

		}

		emit updateToggleAction(TRUE);
		updateMultiSearchTools(FALSE, TRUE);
		updateEntryList2(matches);
		displayMode2 = DISP_LIST;
		toggleContentForce2 (DISP_LIST_AND_CONTENT);

		current_bk_pos = current_pos;

    }
}

void
Zten::addMultiSearchEntry (QBookEntry *be, int index, QString label,
			   bool have_candidate)
{
  current_book = be;
  multi_number++;
  switch (index) {
  case 0:
      MultiLabel1->setText (label);
      LineEdit1->setEnabled (TRUE);
      if (have_candidate)
		  MultiCandidate1->setEnabled (TRUE);
      break;
  case 1:
      MultiLabel2->setText (label);
      LineEdit2->setEnabled (TRUE);
      if (have_candidate)
		  MultiCandidate2->setEnabled (TRUE);
	  MultiLabel6->setText(label);
      break;
  case 2:
      MultiLabel3->setText (label);
      LineEdit3->setEnabled (TRUE);
      if (have_candidate)
		  MultiCandidate3->setEnabled (TRUE);
      break;
  case 3:
      MultiLabel4->setText (label);
      LineEdit4->setEnabled (TRUE);
      if (have_candidate)
		  MultiCandidate4->setEnabled (TRUE);
      break;
  case 4:
      MultiLabel5->setText (label);
      LineEdit5->setEnabled (TRUE);
      if (have_candidate)
		  MultiCandidate5->setEnabled (TRUE);
      break;
  default:
      break;
  }
}

void
Zten::addMultiSearch (QBookEntry *be, int multi_id, QString title, int)
{
  MultiSearchCombo->insertItem (be->getTitle () + " " + title);
  ms_list.append (new QBookMultiSearch (be, multi_id));	
}

void
Zten::findMultiSearches ()
{
  QBookEntry *be;

  MultiSearchCombo->clear ();
  ms_list.clear ();
  for (be = books.first (); be != NULL; be = books.next ())
    {
      be->findMultiSearch ();
    }
}

void
Zten::findMenues ()
{
  QBookEntry *be;

  MenuSearch->clear ();
  menu_books.clear ();
  for (be = books.first (); be != NULL; be = books.next ())
    {
      if (eb_have_menu (be->getBook ()))
	  {
		  MenuSearch->insertItem (be->getTitle ());
		  menu_books.append (be);
	  }
    }
}

void
Zten::menuSelected (int index)
{
  if (menu_books.count () > 0)
    {
      MenuSearch->setEnabled (TRUE);
      QBookEntry *be = menu_books.at (index);
      current_book = be;
      menu_last_book_ident = be->getIdentity ();
      gotoMenu (be);
    }
  else
    {
      MenuSearch->setEnabled (FALSE);
    }
}

void
Zten::openBooks ()
{
	bool isKakasiExist = TRUE;

	history_current = -1;

	if (!bookOpened && dictList.count()) {


		DictInfo *dictInfo;

		if( !checkKakasiDict() )
			isKakasiExist = FALSE;

		for (dictInfo = dictList.first(); dictInfo != 0;
			 dictInfo = dictList.next()) {

			if(dictInfo->getSelected()) {
				if(!isKakasiExist)
					dictInfo->setExtension("");
				QBookEntry *be = new QBookEntry (dictInfo->getPath(),
												 dictInfo->getOrgName(),
												 dictInfo->getSubpos(),
												 dictInfo->getExtension(),
												 dictInfo->getAppendix(),
												 dictInfo->getDirectory(),
												 colorGroup ());
				if (be->isOpened ())
				{
					be->setFontSize (cur_font.pixelSize ());
					connect (be, SIGNAL (foundMultiSearch(QBookEntry*,int,QString,int)),
							 this, SLOT (addMultiSearch (QBookEntry*,int,QString,int)));
					connect (be, SIGNAL (foundMultiSearchEntry(QBookEntry*,int,
															   QString,bool)),
							 this, SLOT (addMultiSearchEntry (QBookEntry*,int,
															  QString,bool)));
					if (be) books.append(be);
				}
				else
				{
					delete be;
				}
			}
			bookOpened = TRUE;
		}
		setCurrentFont (cur_font_id);
		bool bookmarkshown = false;
		if(isBookMarkShown) {
			bookmarkshown = true;
			isBookMarkShown = false;
		}
		setImageScale (cur_scale);
		if(bookmarkshown)
			isBookMarkShown = true;

    }
}

void
Zten::closeBooks ()
{
	books.clear();
	bookOpened = FALSE;
}

void
Zten::expectCardBooks ()
{
	QList <DictInfo> tmpList;

	DictInfo *dictInfo;

	for (dictInfo = dictList.first(); dictInfo != 0;
		 dictInfo = dictList.next()) {

		DictInfo *tmpInfo = new DictInfo();
		tmpInfo->setPath(dictInfo->getPath());
		tmpInfo->setSubpos(dictInfo->getSubpos());
		tmpInfo->setSelected(dictInfo->getSelected());
		tmpInfo->setTitle(dictInfo->getTitle());
		QString name = dictInfo->getOrgName();
		tmpInfo->setOrgName(name);
		tmpInfo->setName(QString::fromUtf8(name));
		tmpInfo->setMemory(dictInfo->getMemory());
		tmpInfo->setDirectory(dictInfo->getDirectory());

		tmpList.append(tmpInfo);

	}

	dictList.clear();
	closeBooks();

	for (dictInfo = tmpList.first(); dictInfo != 0;
		 dictInfo = tmpList.next()) {

		if(dictInfo->getPath() != "/mnt/card/dict/" &&
		   dictInfo->getPath() != "/mnt/cf/dict/" )
			dictList.append(dictInfo);

	}

	openBooks();

	if (!bookOpened && !dictList.count()) {
		clearLine ();
		return;
	}
	activateAndSearch();		


}

void
Zten::multiResume ()
{

  if (multi_last_book_ident.isNull ())
    {
      MultiSearchCombo->setCurrentItem (0);
      multiSelected (0);
    }
  else
    {
      QBookMultiSearch *ms;
      int count = 0;
      for (ms = ms_list.first (); ms != NULL; ms = ms_list.next ())
	{
	  if (ms->getBookEntry ()->equals (multi_last_book_ident) &&
	      ms->getMultiId () == multi_last_id)
	    {
	      MultiSearchCombo->setCurrentItem (count);
	      multiSelected (count);
	      return;
	    }
	  count++;
	}
      multiSelected (0);
    }
}

void
Zten::menuResume ()
{
  if (menu_last_book_ident.isNull ())
    {
      MenuSearch->setCurrentItem (0);
      menuSelected (0);
    }
  else
    {
      QBookEntry *be;
      int count = 0;
      for (be = menu_books.first (); be != NULL; be = menu_books.next ())
	{
	  if (be->equals (menu_last_book_ident))
	    {
	      MenuSearch->setCurrentItem (count);
	      menuSelected (count);
	      return;
	    }
	  count++;
	}
      menuSelected (0);
    }  
}


void
Zten::paste()
{
  QString str = QApplication::clipboard()->text();

  if(str.isEmpty())
	 return;

  if(currentMethod == MENU_SEARCH_METHOD_NO){
	  return;
  }
  else if(currentMethod == MAIN_SEARCH_METHOD_NO ||
		   currentMethod == CONDITION_SEARCH_METHOD_NO ||
		   currentMethod == COMBINATION_SEARCH_METHOD_NO ) {
	  LineEdit->setEditText(str);
  }
  else {
	  QWidget *widget = focusWidget();
	  if (widget->inherits( "QLineEdit")){
		  ((QLineEdit *)widget)->setText(str);
	  }
  }
}

void
Zten::copy()
{

	QWidget *widget = focusWidget();

	if (widget->inherits( "QLineEdit")){
		((QLineEdit*)widget)->copy();
	} else if ( widget->inherits("SLTextBrowser") ) {
		((SLTextBrowser*)widget)->copy();
	}

}

void Zten::backToMultiCond()
{

	if(currentMethod != MULTI_SEARCH_METHOD_NO)
		return;

	updateMultiSearchTools(TRUE, TRUE);

	if(isGaijiDisp) {
		MultiLabel6->show();
		gaijiBrowser->show();
	}

	EntryList2->hide();
	ContentBrowser2->hide();

	LineEdit1->setFocus();

	history_current = -1;
	emit noHistory();

	emit updateDownAction(FALSE);
	emit updateBookMarkAction(FALSE);
	emit updateToggleAction(FALSE);

}

void Zten::updateLayout()
{
	switch(currentMethod) {
	case MAIN_SEARCH_METHOD_NO: // main search
		changeMainSearchLayout(TRUE);
		clearLine();
		break;
	case MULTI_SEARCH_METHOD_NO: // multi search
		backToMultiCond();
		LineEdit1->setText("");
		LineEdit2->setText("");
		LineEdit3->setText("");
		LineEdit4->setText("");
		LineEdit5->setText("");
		break;
	case CONDITION_SEARCH_METHOD_NO: // condition search
		changeConditionSearchLayout(TRUE);
		clearLine();
		break;
	case COMBINATION_SEARCH_METHOD_NO: // combination search
		changeCombinationSearchLayout(TRUE);
		clearLine();
		break;
	case MENU_SEARCH_METHOD_NO: // menu search
		changeMenuSearchLayout(TRUE);
		break;
	default:
		return;
	}
}

void Zten::updateMultiLayout()
{
	if(currentMethod == MULTI_SEARCH_METHOD_NO ) {
		backToMultiCond();
		LineEdit1->setText("");
		LineEdit2->setText("");
		LineEdit3->setText("");
		LineEdit4->setText("");
		LineEdit5->setText("");
	}
}

void Zten::searchMethodChanged(int no)
{
	if(no == currentMethod)
		return;

	isBookMarkShown = FALSE;

	int before = currentMethod;
	currentMethod = no;

	changeLayout(before);
}

void Zten::setCurrentMethod(int no)
{
	SearchMethodComboBox->setCurrentItem(no);
}

int Zten::getSearchType()
{
	QString selected = SearchTypeButton->text();
	int type = SEARCH_HEAD;

	if(selected == STR_SEARCH_TAIL)
		type = SEARCH_TAIL;
	else if(selected == STR_SEARCH_EXACT)
		type = SEARCH_EXACT;

	return (type - 1);
}

void Zten::searchTypeSelected(int id)
{
	if(currentMethod != COMBINATION_SEARCH_METHOD_NO) {
		SearchTypeButton->setText(SearchTypePopup->text(id));
	}
	else {
		bool b = SearchTypePopup->isItemChecked(id);
		SearchTypePopup->setItemChecked (id, !b);

		combiSearchType = 0;

		if( SearchTypePopup->isItemChecked(SEARCH_HEAD) )
			combiSearchType = 0x01;
		if( SearchTypePopup->isItemChecked(SEARCH_TAIL) )
			combiSearchType += 0x02;
		if( SearchTypePopup->isItemChecked(SEARCH_COND) )
			combiSearchType += 0x04;
		if( !SearchTypePopup->isItemChecked(SEARCH_HEAD) &&
			!SearchTypePopup->isItemChecked(SEARCH_TAIL) &&
			!SearchTypePopup->isItemChecked(SEARCH_COND) ) {
			SearchTypePopup->setItemChecked (SEARCH_HEAD, TRUE);
			combiSearchType = 0x01;
		}

		setupSearchTypeButton_combi();

	}

}


void Zten::changeLayout(int before)
{
	// disable old layout
	switch(before) {
	case MAIN_SEARCH_METHOD_NO: // main search
		changeMainSearchLayout(FALSE);
		break;
	case MULTI_SEARCH_METHOD_NO: // multi search
		changeMultiSearchLayout(FALSE);
		break;
	case CONDITION_SEARCH_METHOD_NO: // condition search
		changeConditionSearchLayout(FALSE);
		break;
	case COMBINATION_SEARCH_METHOD_NO: // combination search
		changeCombinationSearchLayout(FALSE);
		break;
	case MENU_SEARCH_METHOD_NO: // menu search
		changeMenuSearchLayout(FALSE);
		break;
	default:
		return;
	}

	// enable new layout
	switch(currentMethod) {
	case MAIN_SEARCH_METHOD_NO: // main search
		changeMainSearchLayout(TRUE);
		break;
	case MULTI_SEARCH_METHOD_NO: // multi search
		changeMultiSearchLayout(TRUE);
		break;
	case CONDITION_SEARCH_METHOD_NO: // condition search
		changeConditionSearchLayout(TRUE);
		break;
	case COMBINATION_SEARCH_METHOD_NO: // combination search
		changeCombinationSearchLayout(TRUE);
		break;
	case MENU_SEARCH_METHOD_NO: // menu search
		changeMenuSearchLayout(TRUE);
		break;
	default:
		return;
	}

	current_entrylist->clear();
	ContentBrowser->setText ("");

	emit updateBookMarkAction(FALSE);
}


void Zten::changeMainSearchLayout(bool b)
{

	if(b) {
		DelButton->setEnabled(TRUE);
		LineEdit->setEnabled(TRUE);

		CondButton->hide();
		DelButton->show();
		displayMode = DISP_LIST;

		EntryList->show();

		SearchTypePopup->setCheckable (false);

		history.resize (0);
		history_current = -1;
		emit noHistory ();
		current_entrylist = EntryList;
		current_browser = ContentBrowser;
		ContentBrowser->setText ("");
		current_book = NULL;
		EntryList->clear ();

		openBooks ();
		emit contentSelected ();
		emit updateDownAction(FALSE);

		LineEdit->setFocus();
	}
	else {
		EntryList->hide();
		ContentBrowser->hide();
	}
}

void Zten::changeMultiSearchLayout(bool b)
{
	if(b) {
		DelButton->setEnabled(FALSE);
		LineEdit->setEnabled(FALSE);

		isGaijiDisp = FALSE;
		updateMultiSearchTools(TRUE, TRUE);

		SearchTypePopup->setCheckable (false);

		history.resize (0);
		history_current = -1;
		emit noHistory ();
		current_entrylist = EntryList2;
		current_browser = ContentBrowser2;
		ContentBrowser2->setText ("");
		current_book = NULL;
		EntryList2->clear ();
		openBooks ();
		emit contentSelected ();

		emit updateDownAction(FALSE);

		findMultiSearches ();
		multiResume ();
		LineEdit1->setFocus();

	}
	else {
		CondButton->hide();
		DelButton->show();

		updateMultiSearchTools(FALSE, FALSE);

		EntryList2->hide();
		ContentBrowser2->hide();
	}

	emit updateToggleAction(!b);

	SearchTypeButton->setEnabled (!b);

}

void Zten::changeConditionSearchLayout(bool b)
{

	changeMainSearchLayout(b);

	SearchTypeButton->setEnabled (!b);
}

void Zten::setupSearchTypeButton_combi()
{
		switch(combiSearchType) {
		case 0x01:
			SearchTypeButton->setText(SearchTypePopup->text(SEARCH_HEAD));
			break;
		case 0x02:
			SearchTypeButton->setText(SearchTypePopup->text(SEARCH_TAIL));
			break;
		case 0x03:
		case 0x05:
		case 0x07:
			SearchTypeButton->setText(STR_SEARCH_HEAD_P);
			break;
		case 0x04:
			SearchTypeButton->setText(SearchTypePopup->text(SEARCH_COND));
			break;
		case 0x06:
			SearchTypeButton->setText(STR_SEARCH_TAIL_P);
			break;
		}

}

void Zten::initSearchType_combi()
{
	if( combiSearchType == 0){
		SearchTypePopup->setItemChecked(SEARCH_HEAD, TRUE);
		SearchTypeButton->setText(SearchTypePopup->text(SEARCH_HEAD));
	}
	else {
		if(combiSearchType & 0x01)
			SearchTypePopup->setItemChecked(SEARCH_HEAD, TRUE);
		if(combiSearchType & 0x02)
			SearchTypePopup->setItemChecked(SEARCH_TAIL, TRUE);
		if(combiSearchType & 0x04)
			SearchTypePopup->setItemChecked(SEARCH_COND, TRUE);
		setupSearchTypeButton_combi();
	}

}

void Zten::changeCombinationSearchLayout(bool b)
{
	changeMainSearchLayout(b);

	if(b) {
		findMultiSearches ();
		multiResume ();

		resumeType = getSearchType() + 1;

		SearchTypePopup->setCheckable (true);
		SearchTypePopup->clear();
		SearchTypePopup->insertItem(STR_SEARCH_HEAD, SEARCH_HEAD);
		SearchTypePopup->insertItem(STR_SEARCH_TAIL, SEARCH_TAIL);
		SearchTypePopup->insertItem(STR_SEARCH_COND, SEARCH_COND);

		initSearchType_combi();

	}
	else {
		SearchTypePopup->clear();
		SearchTypePopup->insertItem(STR_SEARCH_HEAD, SEARCH_HEAD);
		SearchTypePopup->insertItem(STR_SEARCH_TAIL, SEARCH_TAIL);
		SearchTypePopup->insertItem(STR_SEARCH_EXACT, SEARCH_EXACT);

		SearchTypeButton->setText(SearchTypePopup->text(resumeType));

	}
}

void Zten::changeMenuSearchLayout(bool b)
{
	if(b) {
		CondButton->hide();
		DelButton->show();

		DelButton->setEnabled(FALSE);
		LineEdit->setEnabled(FALSE);

		MenuSearch->show();
		ContentBrowser3->show();

		history.resize (0);
		history_current = -1;
		emit noHistory ();
		current_browser = ContentBrowser3;
		ContentBrowser3->setText ("");
		current_book = NULL;

		openBooks ();
		emit contentSelected ();

		findMenues ();
		menuResume ();
		ContentBrowser3->setFocus();
	}
	else {
		MenuSearch->hide();
		ContentBrowser3->hide();
		ContentBrowser3->setText ("");
	}

	emit updateToggleAction(!b);

	SearchTypeButton->setEnabled (!b);

}

bool Zten::isTextSelected()
{
	bool ret = FALSE;

	QWidget *widget = focusWidget();

	if ( widget->inherits("QLineEdit")){
		if (((QLineEdit *)widget)->hasMarkedText()) {
			ret = TRUE;
		}
	} else if ( widget->inherits("SLTextBrowser") ) {
		if (((SLTextBrowser *)widget)->hasSelectedText())
			ret = TRUE;
	}

	return ret;
}

bool Zten::isCurrentMainSearch()
{
	if(currentMethod == MAIN_SEARCH_METHOD_NO)
		return TRUE;
	return FALSE;

}

bool Zten::isCurrentMenuSearch()
{
	if(currentMethod == MENU_SEARCH_METHOD_NO)
		return TRUE;
	return FALSE;

}

bool Zten::isCurrentMultiSearchInput()
{
	if( (currentMethod == MULTI_SEARCH_METHOD_NO) &&
		!MultiSearchCombo->isHidden() )
		return TRUE;
	return FALSE;

}

bool Zten::hasSearchResult()
{
	if(currentMethod != MENU_SEARCH_METHOD_NO) {
		if(current_entrylist->count() == 0)
			return FALSE;
		else if(current_entrylist->count() == 1){
			if(current_entrylist->text(0) == tr("No matches"))
				return FALSE;
		}
	}

	return TRUE;

}

bool Zten::isShownBookMark()
{

	return isBookMarkShown;
}

void Zten::setBrowserHScrollMode(bool b)
{

	if(b) {
		ContentBrowser->setHScrollBarMode( SLTextBrowser::Auto );
		ContentBrowser2->setHScrollBarMode( SLTextBrowser::Auto );
	}
	else {
		ContentBrowser->setHScrollBarMode( SLTextBrowser::AlwaysOff );
		ContentBrowser2->setHScrollBarMode( SLTextBrowser::AlwaysOff );
	}

}

void Zten::updateMultiSearchTools(bool isToolsDisp, bool isButtonChange)
{
	if(isButtonChange) {
		if(isToolsDisp) {
			CondButton->hide();
			DelButton->show();
		}
		else {
			DelButton->hide();
			CondButton->show();
		}
	}

	if(isToolsDisp) {
		MultiSearchCombo->show();
		MultiLabel1->show();
		MultiLabel2->show();
		MultiLabel3->show();
		MultiLabel4->show();
		MultiLabel5->show();
		LineEdit1->show();
		LineEdit2->show();
		LineEdit3->show();
		LineEdit4->show();
		LineEdit5->show();
		MultiCandidate1->show();
		MultiCandidate2->show();
		MultiCandidate3->show();
		MultiCandidate4->show();
		MultiCandidate5->show();

		MultiLabel6->hide();
		gaijiBrowser->hide();

		gaijiBrowser->setEnabled(FALSE);
	}
	else {
		MultiSearchCombo->hide();
		MultiLabel1->hide();
		MultiLabel2->hide();
		MultiLabel3->hide();
		MultiLabel4->hide();
		MultiLabel5->hide();
		LineEdit1->hide();
		LineEdit2->hide();
		LineEdit3->hide();
		LineEdit4->hide();
		LineEdit5->hide();
		MultiCandidate1->hide();
		MultiCandidate2->hide();
		MultiCandidate3->hide();
		MultiCandidate4->hide();
		MultiCandidate5->hide();

		MultiLabel6->hide();
		gaijiBrowser->hide();

		ContentBrowser2->setText("");
	}
}

QString Zten::expectIrregularWord(QString str)
{
	QString retStr = str;

	retStr.replace(QString(" "), QString(""));
	retStr.replace(QString("/"), QString(""));
	retStr.replace(QString("\'"), QString(""));
	retStr.replace(QString("-"), QString(""));
	retStr.replace(tr("HankakuChuten"), QString(""));

	retStr.replace(tr("ZenkakuChuten"), QString(""));
	retStr.replace(tr("ZenkakuApostrophe"), QString(""));
	retStr.replace(tr("ZenkakuSlush"), QString(""));
	retStr.replace(tr("ZenkakuHyphen"), QString(""));

	int pos = 0;
	while(pos <= (int)retStr.length()) {
		pos = retStr.find(".",pos);
		if(pos<1)
			break;
		retStr.remove(pos,1);
	}

	return retStr;
}

void Zten::checkBooks()
{

	DictInfo *dictInfo;

	QList <DictInfo> tmpList;

	closeBooks();

	isBookMarkShown = false;

	for (dictInfo = dictList.first(); dictInfo != 0;
		 dictInfo = dictList.next()) {

		QString path = dictInfo->getPath()+dictInfo->getName();
		QDir dir2(path);
		if ( !dir2.exists() ) {

			QString title = dictInfo->getTitle();
			QMessageBox::warning(this, QObject::tr("Error"),
								 "[" + title + "]\n" + tr("not exist"));
			tmpList.append(dictInfo);
		}
	}

	for (dictInfo = tmpList.first(); dictInfo != 0;
		 dictInfo = tmpList.next()) {

		dictList.remove(dictInfo);
	}

	openBooks();
}

void Zten::reExec()
{
	LineEdit->setEditText ("");
	changeLayout(currentMethod);
}

bool Zten::setCurrentBkPos (SlListBoxItem *item)
{
	if(isBookMarkShown)
		return TRUE;

	if (item) {
		QBookHitItem *bi = (QBookHitItem *)item;
		if (bi == NULL)
			return FALSE;
		QBookEntry *be;
		be = bi->getBookEntry ();

		current_book = be;

		if (be) {
			current_bk_pos = be->getHitPosition(bi->getHitPos());
			current_headpos = be->getHitHeadPosition(bi->getHitPos());
		}
	}
	return TRUE;
}

void Zten::checkBookMarkDict()
{

	QString bkTitle = current_bk.getBookTitle();

	DictInfo *dictInfo;
	for (dictInfo = dictList.first(); dictInfo != 0;
		 dictInfo = dictList.next()) {

		if( (bkTitle == dictInfo->getTitle()) && dictInfo->getSelected())
			return; // exist bk dict.
	}

	current_entrylist->clear();
	current_browser->setText("");
	toggleContentForce (DISP_LIST);
	
}

void Zten::slotClearMpegFlag()
{
  setPlayMpegFlag(FALSE);
}

#define KANWA_DICT "/home/QtPalmtop/etc/kanwadict"
#define ITAIJI_DICT "/home/QtPalmtop/etc/itaijidict"
bool Zten::checkKakasiDict()
{

	QFile f(KANWA_DICT);
	if( !f.exists() )
		return FALSE;
	
	QFile f2(ITAIJI_DICT);
	if( !f2.exists() )
		return FALSE;

	return TRUE;
}

void Zten::showDlgWait()
{

	SlDlgWait dlg(this,tr("You cannot operate this action"));
	dlg.exec();

	usleep(10*100000);

}
