#ifndef ZTEN_H
#define ZTEN_H

#include <eb.h>
#include <error.h>

#include <stdio.h>
#include <stdlib.h>

#include <qlist.h>
#include <qlistview.h>
#include <qlistbox.h>
#include <qpopupmenu.h>
#include <qfont.h>
#include "qbookentry.h"
#include "qbookitem.h"
#include "sllistboxrichtext.h"
#include "../library/config.h"


class QVBoxLayout; 
class QHBoxLayout; 
class QGridLayout; 
class QBookBrowser;
class QGroupBox; 
class QComboBox;
class QHistoryLineEdit;
class QLabel;
class QLineEdit;
class QListView;
class QListViewItem;
class QPushButton;
class QTabWidget;
class QGroupBox;

enum layoutNo {
	MAIN_SEARCH_METHOD_NO = 0,
	MULTI_SEARCH_METHOD_NO,
	CONDITION_SEARCH_METHOD_NO,
	COMBINATION_SEARCH_METHOD_NO,
	MENU_SEARCH_METHOD_NO
};

enum dispModeNo {
	DISP_LIST = 0,
	DISP_CONTENT,
	DISP_LIST_AND_CONTENT
};

enum searchTypeNo {
	SEARCH_HEAD = 1,
	SEARCH_TAIL,
	SEARCH_EXACT,
	SEARCH_MULTI
};
#define SEARCH_COND SEARCH_EXACT

class DictInfo : public QObject
{ 
	Q_OBJECT

public:
	DictInfo();
	void reset();

	void setPath (QString p) { path = p; }
	void setSubpos (int pos) { subpos = pos; }
	void setSelected (bool b) { selected = b; }
	void setTitle (QString t) { title = t; }
	void setName (QString n) { name = n; }
	void setOrgName (QString n) { orgname = n; }
	void setExtension (QString e) { extension = e; }
	void setAppendix (QString a) { appendix = a; }
	void setMemory (QString m) { memory = m; }
	void setDirectory (QString d) { directory = d; }

	QString getPath () { return path; }
	int getSubpos ()  { return subpos; }
	bool getSelected() { return selected; }
	QString getTitle () { return title; }
	QString getName () { return name; }
	QString getOrgName () { return orgname; }
	QString getExtension () { return extension; }
	QString getAppendix () { return appendix; }
	QString getMemory () { return memory; }
	QString getDirectory () { return directory; }

private:
	QString path;
	int subpos;
	bool selected;
	QString title;
	QString name;
	QString orgname;
	QString extension;
	QString appendix;
	QString memory;
	QString directory;

};

class BookMark : public QObject
{ 
	Q_OBJECT

public:
	BookMark();
	void reset();

	void setEntry(QString e) { entry = e; }
	void setBookTitle(QString b) { book_title = b; }
	void setPosition(EB_Position p) { pos = p; }
	void setHeadPosition(EB_Position p) { head_pos = p; }

	QString getEntry() { return entry; }
	QString getBookTitle() { return book_title; }
	EB_Position getPosition() { return pos; }
	EB_Position getHeadPosition() { return head_pos; }

private:
	QString entry;
	QString book_title;
	EB_Position pos;
	EB_Position head_pos;
};


class QBookHitItem : public SlListBoxRichText
{
 public:
  QBookHitItem (const QString &label, QBookEntry *be, int hit_pos);
  ~QBookHitItem ();
  QBookEntry *getBookEntry () { return be; }
  int getHitPos () { return hit_pos; }
 private:
  QBookEntry *be;
  int hit_pos;
};

class QBookMultiSearch
{
 public:
  QBookMultiSearch (QBookEntry *be,int multi_id)
  { this->be = be; this->multi_id = multi_id; }
  ~QBookMultiSearch () {};
  QBookEntry *getBookEntry () { return be; }
  int getMultiId () { return multi_id; }
 private:
  QBookEntry *be;
  int multi_id;
};

//class Zten : public ZtenBase
class Zten : public QWidget
{
  Q_OBJECT
public:
  Zten (QWidget *parent = 0, const char* name = 0, WFlags fl = 0);
  ~Zten ();

  void init();
  void initMainSearch();
  void initMultiSearch();
  void initMenuSearch();

  void setCurrentFont (QFont font);
  void setCurrentFont (int id);
  void setImageScale (int scale);
  void addBook (QBookItem *item);
  void clearBooks ();
  void readConfig ();
  void writeConfig ();
  void lookup (QString key);

  int getCurFontId() { return cur_font_id; }
  int getCurScaleId() { return cur_scale; }
  void keyPressEvent( QKeyEvent *e );
  void changeDispMode (int);
  void changeDispContent(int);
  void changeDispContent2(int);

  void changeMainSearchLayout(bool);
  void changeMultiSearchLayout(bool);
  void changeConditionSearchLayout(bool);
  void changeCombinationSearchLayout(bool);
  void changeMenuSearchLayout(bool);

  void changeLayout(int);
  void updateLayout();
  void updateMultiLayout();

  void openBookmark();
  void addBookmark();
  void initBookMark();

  bool isTextSelected();
  bool isCurrentMainSearch();
  bool isCurrentMenuSearch();
  bool isCurrentMultiSearchInput();
  bool hasSearchResult();
  bool isShownBookMark();
  void setPlayMpegFlag(bool b);
  void setNaviFlag(bool b);
  bool keyEscapeEvent();
  void loopContent();
  void expectCardBooks();
  void checkBooks();
  void reExec();
  void delayBuild();
  void clearField ();

public slots:
  void setDictionary();

  void copy ();
  void paste ();
  void back ();
  void forward ();
  void down ();
  void entryListEscape ();
  void entryListUp ();
  void entryListDown ();
  void contentLineUp ();
  void contentLineDown ();
  void contentPageUp ();
  void contentPageDown ();
  void toggleContent ();
  void toggleDirection ();
  // FIXME: ...
  void toggleContent2 ();
  void toggleDirection2 ();
  void contentPageUp2 ();
  void contentPageDown2 ();
  void contentPageUp3 ();
  void contentPageDown3 ();
  void setIncremental (bool);
  void setContinuous (bool);
  void reload ();
  void slotClearMpegFlag();
  
protected:
  void resizeEvent (QResizeEvent* e);
  bool eventFilter (QObject *o, QEvent *e);

private slots:
  void clearLine ();
  void toggleContentForce (int);

  void openBooks ();
  void closeBooks ();
  void normalsearch ();
  void activateAndSearch ();
  void searchIncremental (const QString &word);
  void goTo (EB_Position);
  void itemSelected (SlListBoxItem *item = NULL);
  void playSound (int);
  void playMpeg (int);
  void addMultiSearch (QBookEntry*,int,QString,int);
  void multiSelected (int);
  void addMultiSearchEntry (QBookEntry *,int,QString,bool);
  void multiSearch ();
  void multiResume ();
  void menuResume ();
  void itemSelected2 (SlListBoxItem *item = NULL);
  void toggleContentForce2 (int);
  
  void gotoMenu (QBookEntry *);
  void findMenues ();
  void menuSelected (int);

  void multiCandidate1 ();
  void multiCandidate2 ();
  void multiCandidate3 ();
  void multiCandidate4 ();
  void multiCandidate5 ();
  void setCandidateLabel (int, QString, QString, bool);
  
  void searchMethodChanged(int);
  void backToMultiCond();

  void searchTypeSelected(int);
  void focusInLineEdit(bool);
  void slotTextChanged1( const QString&);
  void slotTextChanged2( const QString&);
  void slotTextChanged3( const QString&);
  void slotTextChanged4( const QString&);
  void slotTextChanged5( const QString&);
  void linkJump3(EB_Position);

signals:
  // history signal
  void noHistory (); // disable back, forward
  void beginningOfHistory (); // disable back
  void endOfHistory (); // disable forward
  void makeHistory (); // enable back, disable forward
  void followHistory (); // enable back, forward
  // tab signal
  void contentSelected ();
  void incrementalIsSet (bool);

  void continuousIsSet (bool);

  void zoomSelected(bool);
  void updateDownAction(bool);
  void updateBookMarkAction(bool);

  void checkBackNavi();
  void checkForwardNavi();
  void dispContentChanged(int);
  void updateToggleAction(bool);

private:

  Config *config;
  QList<QBookEntry> books;
  QList<QBookMultiSearch> ms_list; // current multi search list.
  QList<QBookEntry> menu_books;

  bool displaying;
  bool incremental_mode;
  bool continuous_mode;
  
  // FIXME: QBookBrowser should have these member.
  QBookEntry *current_book;
  QArray<EB_Position> history;
  int history_current;

  bool goToNoHistory (EB_Position pos);
  QFont cur_font;
  int cur_scale;
  bool redisplay (SlListBoxItem *item = NULL);
  bool bookOpened;
  void linkJump (int pos);
  void findMultiSearches ();
  int multi_number;

  QBookBrowser *current_browser;
  
  SlListBox* current_entrylist;

  // FIXME: ...
  bool displaying2;

  QString candidate1;
  QString candidate2;
  QString candidate3;
  QString candidate4;
  QString candidate5;

  QString multi_last_book_ident;
  int multi_last_id;

  QString menu_last_book_ident;
  
  int cur_font_id;
  int displayMode;
  int displayMode2;

  QList <DictInfo> dictList;
  QList <BookMark> markList;
  EB_Position current_headpos;

  EB_Position current_pos;
  EB_Position current_bk_pos;

  void setCurrentMethod(int);
  bool keyRightEvent();
  bool keyLeftEvent();

  void combinationSearch ();
  int search_normal(QBookEntry *be, QString word, int type);
  int search_combination(QBookEntry *be, QString word, int type, int counter);
  int getSearchType();

  void initSearchType_combi();
  void setupSearchTypeButton_combi();

  void updateMultiSearchTools(bool isToolsDisp, bool isButtonChange);
  void updateEntryList1(int matchNum);
  void updateEntryList2(int matchNum);
  void setBrowserHScrollMode(bool b);
  QString expectIrregularWord(QString str);

  void bookmark_search(BookMark *bk);
  QString convertChuten(QString str);
  QString getSelectedText();
  bool backAction ();
  QString checkGaijiStr(QString text);
  void setCurrentBookMark(BookMark *bk);
  bool setCurrentBkPos(SlListBoxItem *item);
  void checkBookMarkDict();
  bool checkKakasiDict();

  bool menu_goToNoHistory(EB_Position pos, bool isWaitDisp);
  void menu_goTo(EB_Position pos, bool isWaitDisp);
  void linkJump_menu (int pos);

  void showDlgWait();
  void multiSearchTextChanged(const QString& text, int id);

  int currentMethod;
  int resumeType;
  int combiSearchType;
  bool isDictChanged;
  bool isBookMarkShown;
  bool hasFocusLineEdit;
  bool playMpegFlag;
  bool isNaviEnable;
  int backCount;
  bool isGaijiDisp;
  BookMark current_bk;
  bool isSearching;
  bool isSetCandidate;

    QPushButton* DelButton;
	QPushButton* CondButton;
    QHistoryLineEdit* LineEdit;
    QComboBox* SearchMethodComboBox;
    QComboBox* SearchTypeComboBox;
    QPushButton* LookupButton;
    SlListBox* EntryList;
    QBookBrowser* ContentBrowser;

    QGridLayout* ZtenBaseLayout;
    QHBoxLayout* MainLayout;
    QHBoxLayout* BookContentLayout;


    QPushButton* SearchTypeButton;
	QPopupMenu* SearchTypePopup;

	////////////////
	QGroupBox* GroupBox;

    QComboBox* MultiSearchCombo;
    QLabel* MultiLabel1;
    QLineEdit* LineEdit1;
    QPushButton* MultiCandidate1;
    QLabel* MultiLabel2;
    QLineEdit* LineEdit2;
    QPushButton* MultiCandidate2;
    QLabel* MultiLabel3;
    QLineEdit* LineEdit3;
    QPushButton* MultiCandidate3;
    QLabel* MultiLabel4;
    QLineEdit* LineEdit4;
    QPushButton* MultiCandidate4;
    QLabel* MultiLabel5;
    QLineEdit* LineEdit5;
    QPushButton* MultiCandidate5;
    SlListBox* EntryList2;
    QBookBrowser* ContentBrowser2;

    QVBoxLayout* GroupBoxLayout;
    QGridLayout* MultiSearchTabLayout;

    QHBoxLayout* Layout_m1;
    QHBoxLayout* Layout_m2;
    QHBoxLayout* Layout_m3;
    QHBoxLayout* Layout_m4;
    QHBoxLayout* Layout_m5;
    QHBoxLayout* BookContentLayout2;

    QComboBox* MenuSearch;
    QBookBrowser* ContentBrowser3;
    QVBoxLayout* MenuTabLayout;

    QLabel* MultiLabel6;
    QHBoxLayout* Layout_m6;
	QBookBrowser* gaijiBrowser;

	BookMark * bookmark;

};

#endif // ZTEN_H
