#include <dirent.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <qfile.h>
#include <qlistview.h>
#include <qcheckbox.h>
#include <qlayout.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qpushbutton.h>
#include <qapplication.h>
#include <resource.h>

#include "zten.h"
#include "ztensetdict.h"
#include "ztenbook.h"
#include "qbookitem.h"
#include "zpixmap.h"

ZtenBook::ZtenBook (QWidget* parent,
		    const char* name, bool modal, WFlags fl) : 
  ZtenBookBase(parent, name, modal, fl)
{
  codec = QTextCodec::codecForName("eucJP");

  ZtenBookBaseLayout->setStretchFactor(CandidateListView, 1);
  ZtenBookBaseLayout->setStretchFactor(SelectedListView, 1);

  // XXX Hack for stupid layout manager.
#if 0
  QWidget *d = QApplication::desktop();
  SelectedListView->setMinimumSize(QSize((int) d->width() * 45 / 100,
					 (int) d->height () * 4 / 5)); 
#endif

  AddButton->setPixmap (Resource::loadPixmap("zdict/arrow_right"));
  DeleteButton->setPixmap (Resource::loadPixmap("zdict/arrow_left"));

  connect ((QObject *)AddButton, SIGNAL (clicked()), this, SLOT (add ()));
  connect ((QObject *)DeleteButton, SIGNAL (clicked()), this, SLOT (del ()));
  connect ((QObject *)OkButton, SIGNAL (clicked()), this, SLOT (accept ()));
  connect ((QObject *)CancelButton, SIGNAL (clicked()), this, SLOT (reject ()));
  
  SelectedListView->setSorting(-1);

  collectBooks ();
  
}

ZtenBook::~ZtenBook ()
{
}

void
ZtenBook::add ()
{
  QBookItem *item = (QBookItem *)CandidateListView->selectedItem ();
  if (item)
    CandidateListView->takeItem ((QListViewItem *)item);
  if (item)
    SelectedListView->insertItem ((QListViewItem *)item);
  if (CandidateListView->currentItem ())
    CandidateListView->setSelected (CandidateListView->currentItem (), TRUE);
}

void
ZtenBook::del ()
{
  QBookItem *item = (QBookItem *)SelectedListView->selectedItem ();
  if (item)
    SelectedListView->takeItem ((QListViewItem *)item);
  if (item)
    CandidateListView->insertItem ((QListViewItem *)item);
}

void
ZtenBook::addSelectedBook (QBookCheckItem *item)
{
  QBookItem *newItem = new QBookItem (SelectedListView);

  newItem->setText (0, item->text (1));
  newItem->setText (1, item->text (2));
  newItem->setOrgName (item->getOrgName());
  newItem->setSubBookPos (item->getSubBookPos ());
  newItem->setPath (item->getPath ());
  newItem->setText (2, item->text (3));
  newItem->setChecked (item->isOn ());
  newItem->setDirectory(item->getDirectory());

  // Move to the bottom;
  QListViewItem *last;
  for (last = SelectedListView->firstChild ();
       last && last->itemBelow ();
       last = last->itemBelow ());
  newItem->moveItem (last);

  QBookItem *delItem = searchBook (item->getPath (),
								   item->text(2),
								   item->getSubBookPos ());
  CandidateListView->removeItem (delItem);
  
}

void
ZtenBook::collectBooks ()
{

  const char *curd;
  int i = 0;
  for (curd = g_book_path[i].path; curd != NULL; curd = g_book_path[++i].path)
    {
      struct dirent **dir;
      int k, j, n_files;

      EB_Book book;
      EB_Subbook_Code subbook_list[EB_MAX_SUBBOOKS];
      int subbook_count;
      char title[EB_MAX_TITLE_LENGTH + 1];
	  char directory[EB_MAX_TITLE_LENGTH + 1];

      if (! (chdir(curd) < 0))
	  {
		  if ((n_files = scandir(".", &dir, 0, alphasort)) < 0)
			  free(dir);
		  else
			  for(k = 0; k < n_files; k++)
			  {
				  if (strcmp(dir[k]->d_name, ".") == 0 || 
					  strcmp(dir[k]->d_name, "..") == 0) 
					  continue;
				  eb_initialize_book (&book);
				  if (eb_bind (&book, dir[k]->d_name) != EB_SUCCESS)
				  {
					  eb_finalize_book (&book);
					  free (dir[k]);
					  continue;
				  }
				  if (eb_subbook_list (&book, subbook_list, &subbook_count)
					  == EB_SUCCESS)
				  {
					  for (j = 0; j < subbook_count; j++)
					  {
						  if (eb_subbook_title2 (&book, subbook_list[j],
												 title) == EB_SUCCESS)
						  {

							  if(!eb_subbook_directory2(&book, subbook_list[j],
														directory) == EB_SUCCESS)
								  continue;

							  QBookItem* item;
							  
							  item = new QBookItem (CandidateListView);
							  item->setText (0, codec->toUnicode(title));
							  QString name = QString::fromUtf8(dir[k]->d_name);
							  item->setText (1, name);
							  item->setOrgName(dir[k]->d_name);
							  item->setSubBookPos(j);
							  item->setPath(curd);
							  item->setText (2, g_book_path[i].name);
							  item->setDirectory(directory);
							  CandidateListView->insertItem (item);
						  }
					  }
				  }
				  eb_finalize_book(&book);
				  free(dir[k]);
			  }
	  }
    }

  curd = getenv("HOME");
  chdir(curd);

}

QBookItem *
ZtenBook::searchBook (const char *path, QString book, int sub_pos)
{
  QListViewItemIterator it (CandidateListView);
  for (; it.current(); ++it)
    {
      QBookItem *bi = (QBookItem *) it.current ();
      if ((strcmp (path, bi->getPath ()) == 0) &&
		  (book.compare(bi->text(1)) == 0) &&
		  bi->getSubBookPos () == sub_pos)
	return bi;
    }
  return NULL;
}

void
ZtenBook::accept ()
{
  QListViewItemIterator it (SelectedListView);
  QBookItem *bi;

  ZtenSetDict *ztenSetDict = (ZtenSetDict *)parent ();

  ztenSetDict->clearBooks ();
  for (; it.current(); ++it)
    {
      bi = (QBookItem *) it.current ();
      ztenSetDict->addBook (bi);
    }

  ZtenBookBase::accept();
}

void ZtenBook::reject ()
{
	QDialog::reject();
}

void ZtenBook::update ()
{
	QListViewItem *item = (QListViewItem *)CandidateListView->firstChild();
	if(item)
		CandidateListView->setSelected(item,TRUE);
}

