#include <dirent.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>


#include <qapplication.h>
#include <qmessagebox.h>
#include <qlistview.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qimage.h>
#include <qpixmap.h>
#include <resource.h>
#include "ztenbookmark.h"
#include "zpixmap.h"

ZtenBookMark::ZtenBookMark (QWidget* parent,
			      const char* name, bool modal, WFlags fl)
    : QDialog( parent, name, modal, fl )
{

    if ( !name )
	setName( "ZtenBookMarkBase" );
    resize( 580, 480 ); 
    setCaption( tr( "BookMark List" ) );
    ZtenBookMarkBaseLayout = new QVBoxLayout( this ); 
    ZtenBookMarkBaseLayout->setSpacing( 6 );
    ZtenBookMarkBaseLayout->setMargin( 11 );

    Layout1 = new QHBoxLayout; 
    Layout1->setSpacing( 6 );
    Layout1->setMargin( 0 );
    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    Layout1->addItem( spacer );

    OpenButton = new QPushButton( this, "OpenButton" );
    OpenButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)5, OpenButton->sizePolicy().hasHeightForWidth() ) );
    OpenButton->setText( tr( "Open" ) );
    Layout1->addWidget( OpenButton );

    DeleteButton = new QPushButton( this, "DeleteButton" );
    DeleteButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)5, DeleteButton->sizePolicy().hasHeightForWidth() ) );
    DeleteButton->setText( tr( "Delete" ) );
    Layout1->addWidget( DeleteButton );
    ZtenBookMarkBaseLayout->addLayout( Layout1 );

    BookMarkList = new SlListBox( this, "BookMarkList" );
    ZtenBookMarkBaseLayout->addWidget( BookMarkList );

	setCaption (tr("ZtenBookMarkList"));

	OpenButton->setPixmap(Resource::loadPixmap("zdict/open_bookmark"));
	DeleteButton->setPixmap(Resource::loadPixmap("trash"));

	connect(OpenButton, SIGNAL( clicked() ),
			this, SLOT( slotOpenBookMark() ) );
	connect(DeleteButton, SIGNAL( clicked() ),
			this, SLOT( slotDeleteBookMark() ) );
	connect (BookMarkList, SIGNAL (returnPressed(SlListBoxItem*)),
			 this, SLOT ( accept()));

	BookMarkList->setFocus ();


}

ZtenBookMark::~ZtenBookMark ()
{
}

void ZtenBookMark::setBookMarkList (QList<BookMark> *list)
{
	BookMark *bk;

	for (bk = list->last(); bk != 0; bk = list->prev()){
		_markList.append(bk);
		BookMarkList->insertItem( new SlListBoxRichText(bk->getEntry(),bk->getEntry()));
	}

	BookMarkList->setCurrentItem(0);
	BookMarkList->setSelected(0, TRUE);

	updateButtons();

}

void ZtenBookMark::updateBookMarkList (QList<BookMark> *list)
{

	BookMark *bk;
	for (bk = _markList.last(); bk != 0; bk = _markList.prev()){
		list->append(bk);
	}

}

void ZtenBookMark::updateButtons ()
{

	int listcount = BookMarkList->count();

	bool isDisp = TRUE;
	if(listcount==0)
		isDisp = FALSE;

	OpenButton->setEnabled(isDisp);
	DeleteButton->setEnabled(isDisp);

}

QString ZtenBookMark::getCurrentEntry ()
{

	if(BookMarkList->count() == 0)
		return QString();

	return BookMarkList->currentText();

}

void ZtenBookMark::slotOpenBookMark ()
{

	accept();
}

void ZtenBookMark::slotDeleteBookMark ()
{

	if (QMessageBox::warning( this, QObject::tr( "Delete" ),
		  tr("Are you sure you want to delete?"),
			QMessageBox::Yes,
			QMessageBox::No|QMessageBox::Default|QMessageBox::Escape, 
			0) != QMessageBox::Yes){
		// Do not delete
		return;
	}

	int current = BookMarkList->currentItem();
	QString delEntry = BookMarkList->currentText();
	SlListBoxItem *cItem = BookMarkList->item(current);
	SlListBoxItem *tItem = cItem->next();
	
	if( tItem == NULL ) {
		tItem = cItem->prev();
	}

	BookMarkList->removeItem(current);

	if(tItem)
		BookMarkList->setSelected(tItem, TRUE);


	BookMark *bk;
	for (bk = _markList.first(); bk != 0; bk = _markList.next()){
		if(delEntry == bk->getEntry()) {
			_markList.remove(bk);
			break;
		}
	}

	updateButtons();
}

void ZtenBookMark::accept ()
{
	QDialog::accept();
}

void ZtenBookMark::reject ()
{
	QDialog::reject();
}

