#include <dirent.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>


#include <qapplication.h>
#include <qmessagebox.h>
#include <qlistview.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qimage.h>
#include <qpixmap.h>
#include <resource.h>

#include "zten.h"
#include "ztensetdict.h"
#include "ztenbook.h"
#include "ztenext.h"
#include "qbookitem.h"
#include "zpixmap.h"


ZtenSetDict::ZtenSetDict (QWidget* parent,
			      const char* name, bool modal, WFlags fl) :
  ZtenSetDictBase(parent, name, modal, fl)			      
{
	DictionaryList->setSorting(-1);

	listChanged = FALSE;

	SelectBookButton->setPixmap(Resource::loadPixmap("zdict/dict_select"));
	DownButton->setPixmap(Resource::loadPixmap("zdict/arrow_down"));
	UpButton->setPixmap(Resource::loadPixmap("zdict/arrow_up"));
	DelButton->setPixmap(Resource::loadPixmap("trash"));

	connect(SelectBookButton, SIGNAL( clicked() ),
			this, SLOT( selectBook() ) );
	connect(DownButton, SIGNAL( clicked() ),
			this, SLOT( listDown() ) );
	connect(UpButton, SIGNAL( clicked() ),
			this, SLOT( listUp() ) );
	connect(DelButton, SIGNAL( clicked() ),
			this, SLOT( deleteBook() ) );
	connect(OkButton, SIGNAL( clicked() ),
			this, SLOT( accept() ) );
	connect(CancelButton, SIGNAL( clicked() ),
			this, SLOT( reject() ) );

	DictionaryList->setFocus();
}

ZtenSetDict::~ZtenSetDict ()
{
}

void ZtenSetDict::clearBooks ()
{
	DictionaryList->clear ();
}

void ZtenSetDict::setDictList (QList<DictInfo> *list)
{
	DictInfo *dict;
	int id = 1;

	for (dict = list->last(); dict != 0;
		 dict = list->prev()){

		QBookCheckItem *newItem = new QBookCheckItem (DictionaryList,
						"",
						QCheckListItem::CheckBox);

		newItem->setOn (dict->getSelected());
		newItem->setText (1, dict->getTitle());
		newItem->setText (2, dict->getName());
		newItem->setOrgName(dict->getOrgName());
		newItem->setSubBookPos (dict->getSubpos());
		newItem->setPath (dict->getPath());
		newItem->setText (3, dict->getMemory());
		newItem->setDid(id);
		newItem->setSortKey(id++);
		newItem->setDirectory(dict->getDirectory());
	}

	DictionaryList->setCurrentItem( DictionaryList->firstChild());
	DictionaryList->setSelected( DictionaryList->firstChild(),TRUE);
}

void ZtenSetDict::updateDictList (QList<DictInfo> *list)
{
	QBookCheckItem *item;

	for(item = (QBookCheckItem*)DictionaryList->firstChild(); item != NULL;
		item = (QBookCheckItem*)item->nextSibling()) {

		DictInfo *dictInfo = new DictInfo();
		dictInfo->setPath(item->getPath());
		dictInfo->setTitle(item->text(1));
		dictInfo->setSelected(item->isOn());
		dictInfo->setName(item->text(2));
		dictInfo->setOrgName(item->getOrgName());
		dictInfo->setMemory(item->text(3));
		dictInfo->setSubpos(item->getSubBookPos());
		dictInfo->setDirectory(item->getDirectory());

		list->append(dictInfo);
	}

}

void ZtenSetDict::checkAvailableDict (QList<DictInfo> *list)
{

	DictInfo *dict;
	QBookCheckItem *item;

	if( (list->count() > 0) && (DictionaryList->firstChild() == NULL) ) {
		listChanged = TRUE;
		return;
	}

	for (dict = list->last(); dict != 0;
		 dict = list->prev()){

		for(item = (QBookCheckItem*)DictionaryList->firstChild();
			item != NULL;
			item = (QBookCheckItem*)item->nextSibling()) {

			if(dict->getTitle() == item->text(1)) {
				if(dict->getSelected() && !item->isOn())
					listChanged = TRUE;
				else if(!dict->getSelected() && item->isOn())
					listChanged = TRUE;
				break;
			}
		}
		if(listChanged)
			break;
	}
}

void ZtenSetDict::addBook(QBookItem *item)
{
	QBookCheckItem *newItem = new QBookCheckItem (DictionaryList,
						"",
						QCheckListItem::CheckBox);

	newItem->setOn (item->getChecked ());
	newItem->setText (1, item->text (0));
	newItem->setText (2, item->text (1));
	newItem->setOrgName (item->getOrgName());
	newItem->setSubBookPos (item->getSubBookPos ());
	newItem->setPath (item->getPath ());
	newItem->setText (3, item->text (2));
	newItem->setDirectory(item->getDirectory());

	// Move to the bottom;
	QListViewItem *last;
	for (last = DictionaryList->firstChild ();
		 last && last->itemBelow ();
		 last = last->itemBelow ());
	newItem->moveItem (last);

	listChanged = TRUE;
}

void ZtenSetDict::setExtension ()
{
	QListViewItem *item = DictionaryList->selectedItem ();
	if (item) {
		ZtenExtension ext (item->text(3), item->text(4), this, 0, true);
		ext.showMaximized ();
		ext.exec ();
    }
}

void ZtenSetDict::selectBook ()
{
	ZtenBook b (this, 0, true);

	QListViewItemIterator it (DictionaryList);
	QBookCheckItem *bi;
	for (; it.current();++it)
    {
		bi = (QBookCheckItem *) it.current ();
		b.addSelectedBook (bi);
    }
	b.showMaximized ();
	b.update();
	b.exec ();  

	if(b.result() == QDialog::Accepted) {
		QListViewItem *item = DictionaryList->firstChild();
		if(item)
			DictionaryList->setSelected(item,TRUE);
	}


}

void ZtenSetDict::listUp ()
{
	QListViewItem *item = DictionaryList->selectedItem ();
	if (item && item->itemAbove ()) {
		item->itemAbove()->moveItem (item);
		listChanged = TRUE;
    }
	DictionaryList->ensureItemVisible (DictionaryList->selectedItem ());  
}

void ZtenSetDict::listDown ()
{
	QListViewItem *item = DictionaryList->selectedItem ();
	if (item && item->itemBelow ()) {
		item->moveItem (item->itemBelow ());
		listChanged = TRUE;
    }
	DictionaryList->ensureItemVisible (DictionaryList->selectedItem ());  
}

void ZtenSetDict::deleteBook ()
{
	QListViewItem *item = DictionaryList->selectedItem ();
	QListViewItem *tmpItem;

	if(!item)
		return;

	if (QMessageBox::warning( this, QObject::tr( "Delete" ),
		  tr("The selected dictionary will be deleted\nAre you sure you want to delete?"),
			QMessageBox::Yes,
			QMessageBox::No|QMessageBox::Default|QMessageBox::Escape, 
			0) != QMessageBox::Yes){
		// Do not delete
		return;
	}

	if (item) {
		if(!(tmpItem = item->itemBelow()) ) {
			tmpItem = item->itemAbove();
		}

		DictionaryList->removeItem (item);
		listChanged = TRUE;

		if(tmpItem)
			DictionaryList->setSelected(tmpItem,TRUE);
	}

}

void ZtenSetDict::accept ()
{
	QDialog::accept();
}

void ZtenSetDict::reject ()
{
	QDialog::reject();
}

