#include <qiconset.h>
#include <qimage.h>
#include <qpixmap.h>
#ifdef QTAPP
#include <qtoolbar.h>
#include <qmenubar.h>
#define QPEToolBar QToolBar
#define QPEMenuBar QMenuBar
#else
#include <qpe/qpetoolbar.h>
#include <qpe/qpemenubar.h>
#endif
#include <qaction.h>
#include <qmenubar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>
#include <qlayout.h>
#include <qfont.h>
#include <qapplication.h>
#include <qclipboard.h>
#ifndef QTAPP
#include <qcopchannel_qws.h>
#include <qcopenvelope_qws.h>
#endif

#include <resource.h>
#include <slmisc.h>
#include <slaboutdialog.h>

#include "ztenwindow.h"
#include "zpixmap.h"
#include "unistd.h"

#define STRVERSION "Ver 1.0"


ZtenWindow::ZtenWindow (QWidget *parent) : QMainWindow (parent)
{
  setCaption (tr("Zdict"));
  setToolBarsMovable(FALSE);
  QPEToolBar *topBar = new QPEToolBar (this);

  topBar->setHorizontalStretchable(TRUE);

  QPEMenuBar *menuBar = new QPEMenuBar (topBar);
  menuBar->setMargin (0);
  QPEToolBar *toolBar;

  toolBar = new QPEToolBar (this);

  editMenu = new QPopupMenu (this);
  menuBar->insertItem (tr("Edit"), editMenu);

  viewMenu = new QPopupMenu (this);
  menuBar->insertItem (tr("View"), viewMenu);

  settingMenu = new QPopupMenu (this);
  settingMenu->setCheckable (true);
  menuBar->insertItem (tr("Setting"), settingMenu);

  // Action
  addBookMarkAction = new QAction (tr ("Add Marker"),
		Resource::loadPixmap("zdict/add_bookmark"), QString::null, 0, this, 0);

  selectBooksAction = new QAction (tr ("Dictinary List"),
		Resource::loadPixmap("zdict/dict_list"), QString::null, 0, this, 0);

  toggleContentAction = new QAction(tr("toggleContent"),
		Resource::loadPixmap("zdict/change_disp"), QString::null, 0, this, 0);
	  
  backAction = new QAction (tr ("Back"),
		Resource::loadPixmap("zdict/arrow_back"), QString::null, 0, this, 0);

  forwardAction = new QAction (tr ("Forward"),
		Resource::loadPixmap("zdict/arrow_forward"), QString::null, 0, this, 0);
  
  downAction = new QAction (tr ("Next"),
		Resource::loadPixmap("zdict/arrow_next"), QString::null, 0, this, 0);

  // ToolBar
  addBookMarkAction->addTo (toolBar);
  selectBooksAction->addTo (toolBar);
  toggleContentAction->addTo (toolBar);
  backAction->addTo (toolBar);
  forwardAction->addTo (toolBar);
  downAction->addTo (toolBar);
  
  // EditMenu
  editMenu->insertItem(Resource::loadPixmap("copy"),
						 tr( "Copy" ), this, SLOT( copy() ), 0, 1 );
  editMenu->insertItem(Resource::loadPixmap("paste"),
						 tr( "Paste" ), this, SLOT( paste() ), 0, 2 );

  // ViewMenu
  viewMenu->insertItem(tr("EntryList"), this, SLOT(slotDispEntry()), 0, 1);
  viewMenu->insertItem(tr("Content"), this, SLOT(slotDispContent()), 0, 2);
  viewMenu->insertItem(tr("EntryAndContent"), this, SLOT(slotDispEntryContent()), 0, 3);
  viewMenu->insertSeparator();

  font = new QPopupMenu(this);
  viewMenu->insertItem (tr("Font"), font, 0, 4);
  SlMisc::insertStandardFontMenu(font);

  scale = new QPopupMenu(this);
  viewMenu->insertItem (tr("Image scale"), scale, 0, 5);
  SlMisc::insertStandardFontMenu(scale);
  viewMenu->insertSeparator();
  
  continuous_id = viewMenu->insertItem( tr( "Continuous" ), this,
										 SLOT( toggleContinuous() ), 0, 6 );
  viewMenu->insertSeparator();
  viewMenu->insertItem(  Resource::loadPixmap("zdict/add_bookmark"),
						 tr( "Add Marker" ), this, SLOT( slotAddMarker() ), 0, 7 );
  viewMenu->insertItem( tr( "Open Marker" ), this, SLOT( slotOpenMarker() ), 0, 8 );

  // SettingMenu
  incremental_id = settingMenu->insertItem (tr("Incremental Search"), this,
					SLOT(toggleIncremental()), 0, 1);
  
  settingMenu->insertItem(  Resource::loadPixmap("zdict/dict_list"),
						 tr( "Dictinary List" ), this, SLOT( setDictionary() ) );

  settingMenu->insertSeparator();
  settingMenu->insertItem( Resource::loadPixmap("help_icon"),
							 tr( "Help" ), this, SLOT( slotHelp() ) );
  settingMenu->insertItem( tr( "About" ), this, SLOT( slotAboutZten() ) );

  zten = new Zten (this);
  setCentralWidget (zten);

  // Receive QCOP message.
  QCopChannel *channel = new QCopChannel( "QPE/Application/zdict", this );
  connect(channel, SIGNAL(received(const QCString&, const QByteArray&)),
	  this, SLOT(dispatchMessage(const QCString&, const QByteArray&)));  

  QCopChannel *sysChannel = new QCopChannel( "QPE/System", this );
  connect( sysChannel, SIGNAL(received(const QCString&, const QByteArray&)),
	  this, SLOT(dispatchMessage(const QCString&, const QByteArray&)) );

  connect( qApp, SIGNAL(appRaised(const QString&)), this, SLOT(slotAppRaised(const QString&)));

  // EditMenu
  connect (editMenu, SIGNAL(aboutToShow()), this, SLOT(updateEditMenu()));

  //ViewMenu
  connect (addBookMarkAction, SIGNAL (activated ()), this, SLOT (slotAddMarker()));
  connect (selectBooksAction, SIGNAL (activated ()), this, SLOT (setDictionary()));
  connect (toggleContentAction, SIGNAL(activated()), this, SLOT(slotToggleContent()) );
  connect (backAction, SIGNAL (activated ()), this, SLOT (back ()));
  connect (forwardAction, SIGNAL (activated ()), this, SLOT (forward ()));
  connect (downAction, SIGNAL (activated ()), this, SLOT (down ()));
  connect (viewMenu, SIGNAL(aboutToShow()), this, SLOT(updateViewMenu()));

  //setting
  connect (font, SIGNAL(activated(int)), this, SLOT(setCurrentFont(int)));
  connect (scale, SIGNAL(activated(int)), this, SLOT(setImageScale(int)));
  connect (settingMenu, SIGNAL(aboutToShow()), this, SLOT(updateSettingMenu()));

  //zten signal
  connect (zten, SIGNAL (noHistory ()), this, SLOT (disableNavigations()));
  connect (zten, SIGNAL (beginningOfHistory ()), this, SLOT (disableBack ()));
  connect (zten, SIGNAL (endOfHistory ()), this, SLOT (disableForward()));
  connect (zten, SIGNAL (makeHistory ()), this, SLOT (enableBackNavigations()));
  connect (zten, SIGNAL (followHistory ()), this, SLOT (enableNavigations()));
  connect (zten, SIGNAL (contentSelected ()), this, SLOT (enableContentOperations()));
  connect (zten, SIGNAL (incrementalIsSet (bool)), this, SLOT (setIncremental(bool)));
  connect (zten, SIGNAL (continuousIsSet (bool)), this, SLOT (setContinuous(bool)));
  connect (zten, SIGNAL (updateDownAction(bool)), this, SLOT (updateDownAction(bool)));
  connect (zten, SIGNAL (updateBookMarkAction(bool)), this, SLOT (updateBookMarkAction(bool)));

  connect (zten, SIGNAL (checkBackNavi ()), this, SLOT (setZtenBackNavi()));
  connect (zten, SIGNAL (checkForwardNavi ()), this, SLOT (setZtenForwardNavi()));
  connect (zten, SIGNAL (dispContentChanged (int)), this, SLOT (setDispModeMenu(int)));
  connect (zten, SIGNAL (updateToggleAction (bool)), this, SLOT (setToggleAction(bool)));

  zten->readConfig ();

  // set font Menu & scale Menu
  cur_font_id = zten->getCurFontId();
  SlMisc::setFontSizeItemChecked(font,cur_font_id);
  connect (zten, SIGNAL(zoomSelected (bool)), this, SLOT (changeZoom(bool)));
  cur_scale_id = zten->getCurScaleId();
  SlMisc::setFontSizeItemChecked(scale,cur_scale_id);

  // layout set & book open
  zten->changeMainSearchLayout(TRUE);
  zten->initBookMark();

  isResume = FALSE;

  m_bTranslation = false;
  m_bCallme = false;
  
  QTimer::singleShot(0,this,SLOT(ztenDelayBuild()));

}

void ZtenWindow::ztenDelayBuild()
{
	zten->delayBuild();
}

void ZtenWindow::slotAppRaised(const QString& appName)
{
	if(appName == "zdict") {
		zten->setPlayMpegFlag(FALSE);
		zten->checkBooks();
		zten->clearField();
	}
}

void
ZtenWindow::dispatchMessage (const QCString &msg, const QByteArray &data)
{
	QDataStream stream (data, IO_ReadOnly);

	if (msg == "lookup(QString)") {
		QString key;
		stream >> key;
		lookup (QString (key));
    }
	else if (msg == "lookup()") {
		lookup ();
    }
    else if(msg == "appForeground(QString)") {
		QString appName;
		stream >> appName;
		if(appName == "zdict") {
			if(isResume) {
			  zten->reExec();
			  isResume = FALSE;
			}

			if(m_bAppForeground) {
			  if(!m_bCallme && isActiveWindow() && zten->isTextSelected()) {
				copy();
			  }
			}

			m_bAppForeground = true;
			if (!m_bCallme) {
			  QCopEnvelope e("QPE/trans_tray", "callHonyaku(QString)");
			  e << appName;
			  m_bCallme = true;
			}
		}
		else {
		  m_bAppForeground = false;

		  zten->writeConfig();
		}
	}
    else if(msg == "ejectCard()") {
		 zten->expectCardBooks();
	}

	else if (msg == "honyaku()") {
	  if (isActiveWindow()) {
		lookup ();
	  }
	  m_bTranslation = false;
	  m_bCallme = false;
	}
}

void
ZtenWindow::closeEvent(QCloseEvent *e)
{
	if(isEscapeKey) {
		if(!zten->keyEscapeEvent()) {
			e->ignore();
			return;
		}
	}

	isResume = TRUE;
	zten->updateMultiLayout();

	e->accept();

	m_bTranslation = false;
	m_bCallme = false;
}

void
ZtenWindow::slotEscKeyPressed(bool isPressed)
{
	isEscapeKey = isPressed;
}

void
ZtenWindow::lookup (QString key)
{
  showMaximized ();
  raise ();
  zten->lookup (key);
}

void
ZtenWindow::lookup ()
{
  showMaximized ();
  raise ();
  QClipboard *tmp= QApplication::clipboard();
  if (tmp) {
	QString key = tmp->text().simplifyWhiteSpace();
	if (prevClip != key && !key.isEmpty()) {
	  QApplication::clipboard()->clear();
	  zten->lookup (key);
	}
  }
  prevClip = "";
}

void
ZtenWindow::resume ()
{
  zten->reload ();
}

void
ZtenWindow::enableNavigations ()
{
  // enable back, forward
  backAction->setEnabled (TRUE);
  forwardAction->setEnabled (TRUE);
}
void
ZtenWindow::enableBackNavigations ()
{
  // enable back, disable forward
  backAction->setEnabled (TRUE);
  forwardAction->setEnabled (FALSE);
}
void
ZtenWindow::disableNavigations ()
{
  // disable back, forward
  backAction->setEnabled (FALSE);
  forwardAction->setEnabled (FALSE);
}
void
ZtenWindow::disableBack ()
{
  // disable back
  backAction->setEnabled (FALSE);
}
void
ZtenWindow::disableForward ()
{
  // disable forward
  forwardAction->setEnabled (FALSE);
}

void
ZtenWindow::updateDownAction(bool b)
{
  downAction->setEnabled(b);
}

void
ZtenWindow::updateBookMarkAction(bool b)
{
  addBookMarkAction->setEnabled(b);
}

void
ZtenWindow::enableContentOperations ()
{
  selectBooksAction->setEnabled (TRUE);
  toggleContentAction->setEnabled (TRUE);
  backAction->setEnabled (FALSE);
  forwardAction->setEnabled (FALSE);
  downAction->setEnabled (TRUE);  
}

void
ZtenWindow::setToggleAction (bool on)
{

  toggleContentAction->setEnabled (on);
}

void
ZtenWindow::setContinuous (bool on)
{

	if (on) {
		viewMenu->setItemChecked (continuous_id, true);
    } else {
		viewMenu->setItemChecked (continuous_id, false);
    }

}

void
ZtenWindow::setDispModeMenu (int id)
{
	viewMenu->setItemChecked (1, false);
	viewMenu->setItemChecked (2, false);
	viewMenu->setItemChecked (3, false);

	viewMenu->setItemChecked (id+1, true);

	if(id == 0) {
		backAction->setEnabled(false);
		forwardAction->setEnabled(false);
		downAction->setEnabled(false);
	}
	else {
		downAction->setEnabled(zten->hasSearchResult());
	}
}


void
ZtenWindow::toggleContinuous ()
{
	if (viewMenu->isItemChecked (continuous_id)) {
		zten->setContinuous (false);
	}else  {
		zten->setContinuous (true);
    }

	zten->updateLayout();

}

void
ZtenWindow::setIncremental (bool on)
{
  if (on)
    {
      settingMenu->setItemChecked (incremental_id, true);
    }
  else
    {
      settingMenu->setItemChecked (incremental_id, false);
    }
}

void
ZtenWindow::toggleIncremental ()
{
  if (settingMenu->isItemChecked (incremental_id))
    {
      zten->setIncremental (false);
    }
  else
    {
      zten->setIncremental (true);
    }
}

void
ZtenWindow::setCurrentFont (int i)
{
    int logicalSize;
	if(i < 0){
		// toggle
		SlMisc::toggleFontSize(font,cur_font_id);
	}else{
		logicalSize = SlMisc::getLogicalSizeFromMenuId(i);
		if(cur_font_id == logicalSize){
			return;
		}
		SlMisc::setFontSizeItemChecked(font,logicalSize);
		cur_font_id = logicalSize;
	}

	zten->setCurrentFont (cur_font_id);
}

void
ZtenWindow::setImageScale (int i)
{
    int logicalSize;
	if(i < 0){
		// toggle
		SlMisc::toggleFontSize(scale,cur_scale_id);
	}else{
		logicalSize = SlMisc::getLogicalSizeFromMenuId(i);
		if(cur_scale_id == logicalSize){
			return;
		}
		SlMisc::setFontSizeItemChecked(scale,logicalSize);
		cur_scale_id = logicalSize;
	}

	zten->setImageScale (cur_scale_id);

}

void
ZtenWindow::setDictionary()
{
  zten->setDictionary();
}

void
ZtenWindow::listDown ()
{

}

void
ZtenWindow::listUp ()
{

}

void
ZtenWindow::setExtension ()
{

}

void
ZtenWindow::deleteBook ()
{

}

void
ZtenWindow::copy ()
{
  zten->copy();
}

void
ZtenWindow::paste ()
{
  zten->paste();
}

void
ZtenWindow::forward ()
{
  zten->forward();
}

void
ZtenWindow::back ()
{
  zten->back();
}

void
ZtenWindow::down ()
{
  zten->down();
}

void
ZtenWindow::setZtenBackNavi ()
{
	zten->setNaviFlag(backAction->isEnabled());

}

void
ZtenWindow::setZtenForwardNavi ()
{
	zten->setNaviFlag(forwardAction->isEnabled());

}

void ZtenWindow::changeZoom (bool isZoomIn)
{
	SlMisc::toggleFontSize(font,cur_font_id,isZoomIn);
	zten->setCurrentFont (cur_font_id);
}

void ZtenWindow::slotToggleContent()
{
	zten->loopContent();
}


void ZtenWindow::slotDispEntry()
{
	zten->changeDispMode(DISP_LIST);
}


void ZtenWindow::slotDispContent()
{
	zten->changeDispMode(DISP_CONTENT);
}


void ZtenWindow::slotDispEntryContent()
{
	zten->changeDispMode(DISP_LIST_AND_CONTENT);
}

void ZtenWindow::slotOpenMarker()
{

	zten->openBookmark();

}

void ZtenWindow::slotAddMarker()
{

	zten->addBookmark();

}

void ZtenWindow::slotHelp()
{

	QPEDecoration *deco = new QPEDecoration();
	deco->help(this);
}

void ZtenWindow::slotAboutZten()
{

	SlAboutDialog dlg (Resource::loadPixmap("Zdict"),
					   Zten::tr("Zdict"),
					   QString(" "),
					   QString(STRVERSION),
					   QString("%1 %2").arg(__DATE__).arg(__TIME__),
					   this);

	dlg.exec();

}

void ZtenWindow::updateEditMenu()
{

  bool copyflag = FALSE;
  bool pasteflag = FALSE;

  if(zten->isTextSelected())
	  copyflag = TRUE;
  
  editMenu->setItemEnabled(1, copyflag);

  if(!zten->isCurrentMenuSearch()) {
	  if (!QApplication::clipboard()->text().isEmpty()){
		  pasteflag = TRUE;
	  }
  }

  editMenu->setItemEnabled(2, pasteflag);

}


void ZtenWindow::updateViewMenu()
{

	bool condflag = TRUE;
	bool contiflag = TRUE;
	bool bookflag = TRUE;

	if( zten->isCurrentMenuSearch() ||
		zten->isCurrentMultiSearchInput()) {
		condflag = FALSE;
		contiflag = FALSE;
		bookflag = FALSE;
	}
	else {
		if(!zten->hasSearchResult()) {
			bookflag = FALSE;
		}
		else if(zten->isShownBookMark()) {
			bookflag = FALSE;
		}
	}

	if(zten->isShownBookMark())
		contiflag = FALSE;

	viewMenu->setItemEnabled(1, condflag);
	viewMenu->setItemEnabled(2, condflag);
	viewMenu->setItemEnabled(3, condflag);
	viewMenu->setItemEnabled(6, contiflag); 

	viewMenu->setItemEnabled(7, bookflag);  // add bookmark

}

void ZtenWindow::updateSettingMenu()
{

  bool flag = FALSE;

  if(zten->isCurrentMainSearch())
	  flag = TRUE;

  settingMenu->setItemEnabled(1, flag);

}

bool ZtenWindow::eventFilter(QObject *object, QEvent *event)
{
  if (event->type() == QEvent::ShowMaximized) {
	if (!m_bAppForeground && !m_bTranslation && !m_bCallme) {
	  m_bTranslation = true;
#ifndef _QVFB
	  {
		prevClip = QApplication::clipboard()->text().simplifyWhiteSpace();

		QCopEnvelope e("QPE/trans_tray", "mainexec()");
	  }
	  qApp->processEvents();
	  ::sleep(1);
#endif
	}
  }
	return QMainWindow::eventFilter(object, event);
}

void ZtenWindow::showEvent(QShowEvent *)
{
	m_bAppForeground = true;
	if (m_bTranslation) return;
	if (m_bCallme) return;
	m_bTranslation = true;
#ifndef _QVFB
	{
	  prevClip = QApplication::clipboard()->text().simplifyWhiteSpace();

		QCopEnvelope qcop("QPE/trans_tray", "mainexec()");
	}
	qApp->processEvents();
#endif
}
