#ifndef ZTENWINDOW_H
#define ZTENWINDOW_H

#include <qwsevent_qws.h>
#include <qpeapplication.h>
#include <qmainwindow.h>
#include "zten.h"

class QAction;
class QPixmap;

class ZtenWindow : public QMainWindow
{
  Q_OBJECT
public:
  ZtenWindow (QWidget *parent = 0);
  void resume ();
private slots:
  void setDictionary();
  void deleteBook ();  
  void copy ();
  void paste ();
  void back ();
  void forward ();
  void down ();
  void listUp ();
  void listDown ();
  void setExtension ();
  void enableNavigations ();     // enable back, forward
  void enableBackNavigations (); // enable back, disable forward
  void disableNavigations (); // disable back, forward
  void disableBack ();	       // disable back
  void disableForward ();     // disable forward

  void setIncremental (bool);
  void toggleIncremental ();

  void setContinuous (bool);
  void toggleContinuous ();

  void enableContentOperations ();
  void setCurrentFont (int);
  void setImageScale (int);

  void dispatchMessage (const QCString &msg, const QByteArray &data);
  void changeZoom (bool isZoomIn);

  void slotToggleContent();
  void slotDispEntry();
  void slotDispContent();
  void slotDispEntryContent();
  void slotOpenMarker();
  void slotAddMarker();
  void slotHelp();
  void slotAboutZten();
  void updateDownAction(bool);
  void updateEditMenu();
  void updateViewMenu();
  void updateSettingMenu();
  void updateBookMarkAction(bool);

  void setZtenBackNavi();
  void setZtenForwardNavi();

  void slotEscKeyPressed(bool);
  void setDispModeMenu(int);
  void setToggleAction(bool);

  void ztenDelayBuild();
  void slotAppRaised (const QString &appName);

protected:
    void closeEvent( QCloseEvent *e );
	bool eventFilter(QObject *object, QEvent *event);
	void showEvent(QShowEvent *e);

private:
  void lookup (QString);
  void lookup ();
 
  QPixmap smallPixmap(const char* xpm[]);
  Zten *zten;
  QPopupMenu *editMenu;
  QPopupMenu *viewMenu;
  QPopupMenu *settingMenu;
  QPopupMenu *font;
  QPopupMenu *scale;
  QAction *listDownAction, *backAction, *forwardAction,
	*extensionAction, *downAction, *listUpAction, 
	*toggleContentAction, *dispEntryAction, 
	*dispContentAction, *dispEntryContentAction,
    *addBookMarkAction, *selectBooksAction, *deleteBookAction;
  int incremental_id;

  int continuous_id;
  int cur_font_id;
  int cur_scale_id;

  bool m_bTranslation;
  bool m_bAppForeground;
  bool m_bCallme;

  bool isEscapeKey;
  bool isResume;

  QString prevClip;
};


class ZdictApl : public QPEApplication {
	Q_OBJECT
public:
	ZdictApl(int & argc,char ** argv) : QPEApplication(argc,argv) {}
	~ZdictApl() {}
#if defined(_WS_QWS_)
	bool qwsEventFilter(QWSEvent *event) {
		if(event->type == QWSEvent::Key &&
		   ((QWSKeyEvent*)event)->simpleData.keycode == Qt::Key_Escape){
			emit escKeyPressed(TRUE);
		}else{
			emit escKeyPressed(FALSE);
		}
		return QPEApplication::qwsEventFilter(event);
	}
#endif

signals:
	void escKeyPressed(bool);
};

#endif // ZTENWINDOW_H
